searchData={"items":[{"type":"module","title":"Patch.Case","doc":"","ref":"Patch.Case.html"},{"type":"module","title":"Patch.Importer","doc":"","ref":"Patch.Importer.html"},{"type":"function","title":"Patch.Importer.all/0","doc":"","ref":"Patch.Importer.html#all/0"},{"type":"module","title":"Patch","doc":"Patch - Ergonomic Mocking for Elixir\n\nPatch makes it easy to mock one or more functions in a module returning a value or executing\ncustom logic.  Patches and Spies allow tests to assert or refute that function calls have been\nmade.\n\nUsing Patch is as easy as adding a single line to your test case.\n\n```elixir\nuse Patch\n```\n\nAfter this all the patch functions will be available, see the function documentation for\ndetails.","ref":"Patch.html"},{"type":"macro","title":"Patch.assert_any_call/1","doc":"Asserts that the given module and function has been called with any arity.\n\n```elixir\npatch(Example, :function, :patch)\n\nassert_any_call Example.function   # fails\n\nExample.function(1, 2, 3)\n\nassert_any_call Example.function   # passes\n```","ref":"Patch.html#assert_any_call/1"},{"type":"function","title":"Patch.assert_any_call/2","doc":"Asserts that the given module and function has been called with any arity.\n\n```elixir\npatch(Example, :function, :patch)\n\nassert_any_call Example, :function   # fails\n\nExample.function(1, 2, 3)\n\nassert_any_call Example, :function   # passes\n```\n\nThis function exists for advanced use cases where the module or function are not literals in the\ntest code.  If they are literals then `assert_any_call/1` should be preferred.","ref":"Patch.html#assert_any_call/2"},{"type":"macro","title":"Patch.assert_called/1","doc":"Given a call will assert that a matching call was observed by the patched function.\n\nThis macro fully supports patterns and will perform non-hygienic binding similar to ExUnit's\n`assert_receive/3` and `assert_received/2`.\n\n```elixir\npatch(Example, :function, :patch)\n\nExample.function(1, 2, 3)\n\nassert_called Example.function(1, 2, 3)   # passes\nassert_called Example.function(1, _, 3)   # passes\nassert_called Example.function(4, 5, 6)   # fails\nassert_called Example.function(4, _, 6)   # fails\n```","ref":"Patch.html#assert_called/1"},{"type":"macro","title":"Patch.assert_called/2","doc":"Given a call will assert that a matching call was observed exactly the number of times provided\nby the patched function.\n\nThis macro fully supports patterns and will perform non-hygienic binding similar to ExUnit's\n`assert_receive/3` and `assert_received/2`.  Any binds will bind to the latest matching call\nvalues.\n\n```elixir\npatch(Example, :function, :patch)\n\nExample.function(1, 2, 3)\n\nassert_called Example.function(1, 2, 3), 1   # passes\nassert_called Example.function(1, _, 3), 1   # passes\n\nExample.function(1, 2, 3)\n\nassert_called Example.function(1, 2, 3), 2   # passes\nassert_called Example.function(1, _, 3), 2   # passes\n```","ref":"Patch.html#assert_called/2"},{"type":"macro","title":"Patch.assert_called_once/1","doc":"Given a call will assert that a matching call was observed exactly once by the patched function.\n\nThis macro fully supports patterns and will perform non-hygienic binding similar to ExUnit's\n`assert_receive/3` and `assert_received/2`.\n\n```elixir\npatch(Example, :function, :patch)\n\nExample.function(1, 2, 3)\n\nassert_called_once Example.function(1, 2, 3)   # passes\nassert_called_once Example.function(1, _, 3)   # passes\n\nExample.function(1, 2, 3)\n\nassert_called_once Example.function(1, 2, 3)   # fails\nassert_called_once Example.function(1, _, 3)   # fails\n```","ref":"Patch.html#assert_called_once/1"},{"type":"function","title":"Patch.debug/1","doc":"Enable or disable library level debugging.\n\nThere is a suite level configuration that can be set by using\n\n```elixir\nconfig :patch,\n  debug: true # or false\n```\n\nCalling this helper will enable or disable debugging for a given test.\n\nLibrary level debugging can be useful when patch behavior isn't meeting expectations.\nAdditional logging will occur at the debug log level using Logger to provide insight into how\nPatch is working.","ref":"Patch.html#debug/1"},{"type":"function","title":"Patch.expose/2","doc":"Expose can be used to turn private functions into public functions for the\npurpose of testing them.\n\nTo expose every private function as a public function, pass the sentinel value `:all`.\n\n```elixir\nexpose(Example, :all)\n```\n\nOtherwise pass a `Keyword.t(arity)` of the functions to expose.\n\nFor example, if one wanted to expose `private_function/1` and `private_function/2`.\n\n```elixir\nexpose(Example, [private_function: 1, private_function: 2])\n```\n\nAfter exposing a function, attempting to call the exposed function will cause the Elixir\nCompiler to flag calls to exposed functions as a warning.  There are companion macros\n`private/1` and `private/2` that test authors can wrap their calls with to prevent warnings.","ref":"Patch.html#expose/2"},{"type":"function","title":"Patch.fake/2","doc":"Fakes out a module with an alternative implementation.\n\nThe real module can still be accessed with `real/1`.\n\nFor example, if your project has the module `Example.Datastore` and there's a fake available in the testing\nenvironment named `Example.Test.InMemoryDatastore`  the following table describes which calls are executed by which\ncode before and after faking with the following call.\n\n```elixir\nfake(Example.Datastore, Example.Test.InMemoryDatastore)\n```\n\n| Calling Code                         | Responding Module before fake/2      | Responding Module after fake/2       |\n|--------------------------------------|--------------------------------------|--------------------------------------|\n| Example.Datastore.get/1              | Example.Datastore.get/1              | Example.Test.InMemoryDatastore.get/1 |\n| Example.Test.InMemoryDatastore.get/1 | Example.Test.InMemoryDatastore.get/1 | Example.Test.InMemoryDatastore.get/1 |\n| real(Example.Datastore).get/1        | (UndefinedFunctionError)             | Example.Datastore.get/1              |\n\nThe fake module can use the renamed module to access the original implementation.","ref":"Patch.html#fake/2"},{"type":"function","title":"Patch.history/2","doc":"Get all the observed calls to a module.  These calls are expressed as a `{name, argument}` tuple\nand can either be provided in ascending (oldest first) or descending (newest first) order by\nproviding a sorting of `:asc` or `:desc`, respectively.\n\n```elixir\nExample.example(1, 2, 3)\nExample.function(:a)\n\nassert history(Example) == [{:example, [1, 2, 3]}, {:function, [:a]}]\nassert history(Example, :desc) == [{:function, [:a]}, {:example, [1, 2, 3]}]\n```\n\nFor asserting or refuting that a call happened the `assert_called/1`, `assert_any_call/2`,\n`refute_called/1`, and `refute_any_call/2` functions provide a more convenient API.","ref":"Patch.html#history/2"},{"type":"function","title":"Patch.inject/4","doc":"","ref":"Patch.html#inject/4"},{"type":"function","title":"Patch.listen/3","doc":"Starts a listener process.\n\nEach listener should provide a unique `tag` that will be used when forwarding messages to the\ntest process.","ref":"Patch.html#listen/3"},{"type":"function","title":"Named Processes - Patch.listen/3","doc":"When used on a named process, this is sufficient to begin intercepting all messages to the named\nprocess.\n\n```elixir\nlisten(:listener, Example)\n```","ref":"Patch.html#listen/3-named-processes"},{"type":"function","title":"Unnamed Processes - Patch.listen/3","doc":"When used on an unnamed process, the process that is spawned will forward any messages to the\ncaller and target process but any processes holding a reference to the old pid will need to be\nupdated.\n\n`replace/3` can be used to replace part of a running process with the listener\n\n```elixir\n{:ok, listener} = listen(:listener, original)\nreplace(target, [:original], listener)\n```\n\n`inject/3` provides a nice convenience when you want to wrap a listener around the pid in the\nstate.\n\n```elixir\ninject(:listener, target, [:original])\n```\n\nThis code will look for the key `:original` in the `target` process's state, and wrap it with\na listener tagged with `:listener`","ref":"Patch.html#listen/3-unnamed-processes"},{"type":"function","title":"Substituting for a Process - Patch.listen/3","doc":"Listeners can also act as a complete substitute for a process.  This is useful in scenarios\nwhere one Process starts other Processes but starting those Processes is outside of the bounds\nof the test.  In those cases you can start a \"targetless listener.\"\n\n`replace/3` can be used to replace part of a running process with the substitute listener.\n\n```elixir\n{:ok, listener} = listen(:listener)\nreplace(target, [:original], listener)\n```\n\n`inject/3` provides a nice conveninece when the state already has a nil that you want to replace\nwith a listener\n\n```elixir\ninject(:listener, target, [:original])\n```\n\nThis code will look for the key `:original` in the `target` process's state, finding it `nil` it\nwill create a \"targetless listener\" tagged with `:listener` and put it in the state.","ref":"Patch.html#listen/3-substituting-for-a-process"},{"type":"function","title":"Patch.patch/3","doc":"Patches a function in a module\n\nWhen called with a function the function will be called instead of the original function and its\nresults returned.\n\n```elixir\npatch(Example, :function, fn arg -> {:mock, arg} end)\n\nassert Example.function(:test) == {:mock, :test}\n```\n\nTo handle multiple arities create a `callable/2` with the `:list` option and the arguments will\nbe wrapped to the function in a list.\n\n```elixir\npatch(Example, :function, callable(fn\n  [] ->\n    :zero\n\n  [a] ->\n    {:one, a}\n\n  [a, b] ->\n    {:two, a, b}\nend, :list))\n\nassert Example.function() == :zero\nassert Example.function(1) == {:one, 1}\nassert Example.function(1, 2) == {:two, 1, 2}\n```\n\nTo provide a function as a literal value to be returned, use the `scalar/1` function.\n\n```elixir\npatch(Example, :function, scalar(fn arg -> {:mock, arg} end))\n\ncallable = Example.function()\nassert callable.(:test) == {:mock, :test}\n```\n\nThe function `cycle/1` can be given a list which will be infinitely cycled when the function is\ncalled.\n\n```elixir\npatch(Example, :function, cycle([1, 2, 3]))\n\nassert Example.function() == 1\nassert Example.function() == 2\nassert Example.function() == 3\nassert Example.function() == 1\nassert Example.function() == 2\nassert Example.function() == 3\nassert Example.function() == 1\n```\n\nThe function `raises/1` can be used to `raise/1` a `RuntimeError` when the function is called.\n\n```elixir\npatch(Example, :function, raises(\"patched\"))\n\nassert_raise RuntimeError, \"patched\", fn ->\n  Example.function()\nend\n```\n\nThe function `raises/2` can be used to `raise/2` any exception with any attributes when the function\nis called.\n\n```elixir\npatch(Example, :function, raises(ArgumentError, message: \"patched\"))\n\nassert_raise ArgumentError, \"patched\", fn ->\n  Example.function()\nend\n```\n\nThe function `sequence/1` can be given a list which will be used until a single value is\nremaining, the remaining value will be returned on all subsequent calls.\n\n```elixir\npatch(Example, :function, sequence([1, 2, 3]))\n\nassert Example.function() == 1\nassert Example.function() == 2\nassert Example.function() == 3\nassert Example.function() == 3\nassert Example.function() == 3\nassert Example.function() == 3\nassert Example.function() == 3\n```\n\nThe function `throws/1` can be given a value to `throw/1` when the function is called.\n\n```elixir\npatch(Example, :function, throws(:patched))\n\nassert catch_throw(Example.function()) == :patched\n```\n\nAny other value will be returned as a literal scalar value when the function is called.\n\n```elixir\npatch(Example, :function, :patched)\n\nassert Example.function() == :patched\n```","ref":"Patch.html#patch/3"},{"type":"macro","title":"Patch.private/1","doc":"Suppress warnings for using exposed private functions in tests.\n\nPatch allows you to make a private function public via the `expose/2` function.  Exposure\nhappens dynamically at test time. The Elixir Compiler will flag calls to exposed functions as a\nwarning.\n\nOne way around this is to change the normal function call into an `apply/3` but this is\ncumbersome and makes tests harder to read.\n\nThis macro just rewrites a normal looking call into an `apply/3` so the compiler won't complain\nabout calling an exposed function.\n\n```elixir\nexpose(Example, :all)\n\npatch(Example, :private_function, :patched)\n\nassert Example.private_function() == :patched   # Compiler will warn about call to undefined function\nassert apply(Example, :private_function, []) == :patched   # Compiler will not warn\nassert private(Example.private_function()) == :patched     # Same as previous line, but looks nicer.\n```","ref":"Patch.html#private/1"},{"type":"macro","title":"Patch.private/2","doc":"Suppress warnings for using exposed private functions in tests.\n\nPatch allows you to make a private function public via the `expose/2` function.  Exposure\nhappens dynamically at test time. The Elixir Compiler will flag calls to exposed functions as a\nwarning.\n\nOne way around this is to change the normal function call into an `apply/3` but this is\ncumbersome and makes tests harder to read.\n\nThis macro just rewrites a normal looking call into an `apply/3` so the compiler won't complain\nabout calling an exposed function, with support for pipelines.\n\n```elixir\nexpose(Example, :all)\n\nexample_that_warns =\n  Example.new()\n  |> Example.private_function()  # Compiler will warn about call to undefined function\n\n\nexample_that_does_not_warn\n  Example.new()\n  |> private(Example.private_function())  # Compiler will not warn and Example.new() is provided\n                                          # as the first argument to Example.private_function/1\n```","ref":"Patch.html#private/2"},{"type":"function","title":"Patch.real/1","doc":"Gets the real module name for a fake.\n\nThis is useful for Fakes that want to defer some part of the functionality back to the real\nmodule.\n\n```elixir\ndef Example do\n  def calculate(a) do\n    # ...snip some complex calculations...\n    result\n  end\nend\n\ndef Example.Fake do\n  import Patch, only: [real: 1]\n\n  def calculate(a) do\n    real_result = real(Example).calculate(a)\n\n    {:fake, real_result}\n  end\nend","ref":"Patch.html#real/1"},{"type":"macro","title":"Patch.refute_any_call/1","doc":"Refutes that the given module and function has been called with any arity.\n\n```elixir\npatch(Example, :function, :patch)\n\nrefute_any_call Example.function   # passes\n\nExample.function(1, 2, 3)\n\nrefute_any_call Example.function   # fails\n```","ref":"Patch.html#refute_any_call/1"},{"type":"function","title":"Patch.refute_any_call/2","doc":"Refutes that the given module and function has been called with any arity.\n\n```elixir\npatch(Example, :function, :patch)\n\nrefute_any_call Example, :function   # passes\n\nExample.function(1, 2, 3)\n\nrefute_any_call Example, :function   # fails\n```\n\nThis function exists for advanced use cases where the module or function are not literals in the\ntest code.  If they are literals then `refute_any_call/1` should be preferred.","ref":"Patch.html#refute_any_call/2"},{"type":"macro","title":"Patch.refute_called/1","doc":"Given a call will refute that a matching call was observed by the patched function.\n\nThis macro fully supports patterns.\n\n```elixir\npatch(Example, :function, :patch)\n\nExample.function(1, 2, 3)\n\nrefute_called Example.function(4, 5, 6)   # passes\nrefute_called Example.function(4, _, 6)   # passes\nrefute_called Example.function(1, 2, 3)   # fails\nrefute_called Example.function(1, _, 3)   # fails\n```","ref":"Patch.html#refute_called/1"},{"type":"macro","title":"Patch.refute_called/2","doc":"Given a call will refute that a matching call was observed exactly the number of times provided\nby the patched function.\n\nThis macro fully supports patterns.\n\n```elixir\npatch(Example, :function, :patch)\n\nExample.function(1, 2, 3)\n\nrefute_called Example.function(1, 2, 3), 2   # passes\nrefute_called Example.function(1, _, 3), 2   # passes\n\nExample.function(1, 2, 3)\n\nrefute_called Example.function(1, 2, 3), 1   # passes\nrefute_called Example.function(1, _, 3), 1   # passes\n```","ref":"Patch.html#refute_called/2"},{"type":"macro","title":"Patch.refute_called_once/1","doc":"Given a call will refute that a matching call was observed exactly once by the patched function.\n\nThis macro fully supports patterns.\n\n```elixir\npatch(Example, :function, :patch)\n\nExample.function(1, 2, 3)\n\nrefute_called_once Example.function(1, 2, 3)   # fails\nrefute_called_once Example.function(1, _, 3)   # fails\n\nExample.function(1, 2, 3)\n\nrefute_called_once Example.function(1, 2, 3)   # passes\nrefute_called_once Example.function(1, _, 3)   # passes\n```","ref":"Patch.html#refute_called_once/1"},{"type":"function","title":"Patch.replace/3","doc":"Convenience function for replacing part of the state of a running process.\n\nUses the `Access` module to traverse the state structure according to the given `keys`.\n\nStructs have special handling so that they can be updated without having to implement the\n`Access` behavior.\n\nFor example to replace the key `:key` in the map found under the key `:map` with the value\n`:replaced`\n\n```elixir\nreplace(target, [:map, :key], :replaced)\n```","ref":"Patch.html#replace/3"},{"type":"function","title":"Patch.restore/1","doc":"Remove any mocks or spies from the given module\n\n```elixir\noriginal = Example.example()\n\npatch(Example, :example, :patched)\nassert Example.example() == :patched\n\nrestore(Example)\nassert Example.example() == original\n```","ref":"Patch.html#restore/1"},{"type":"function","title":"Patch.restore/2","doc":"Remove any patches associated with a function in a module.\n\n```elixir\noriginal = Example.example()\n\npatch(Example, :example, :example_patch)\npatch(Example, :other, :other_patch)\n\nassert Example.example() == :example_patch\nassert Example.other() == :other_patch\n\nrestore(Example, :example)\n\nassert Example.example() == original\nassert Example.other() == :other_patch","ref":"Patch.html#restore/2"},{"type":"function","title":"Patch.spy/1","doc":"Spies on the provided module\n\nOnce a module has been spied on the calls to that module can be asserted / refuted without\nchanging the behavior of the module.\n\n```elixir\nspy(Example)\n\nExample.example(1, 2, 3)\n\nassert_called Example.example(1, 2, 3)   # passes","ref":"Patch.html#spy/1"},{"type":"module","title":"Patch.Listener","doc":"","ref":"Patch.Listener.html"},{"type":"function","title":"Patch.Listener.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Patch.Listener.html#child_spec/1"},{"type":"function","title":"Patch.Listener.init/1","doc":"","ref":"Patch.Listener.html#init/1"},{"type":"function","title":"Patch.Listener.start_link/4","doc":"","ref":"Patch.Listener.html#start_link/4"},{"type":"function","title":"Patch.Listener.target/1","doc":"","ref":"Patch.Listener.html#target/1"},{"type":"type","title":"Patch.Listener.capture_replies_option/0","doc":"Option to control whether or not to capture GenServer.call replies.\n\nDefaults to true","ref":"Patch.Listener.html#t:capture_replies_option/0"},{"type":"type","title":"Patch.Listener.option/0","doc":"Sum-type of all valid options","ref":"Patch.Listener.html#t:option/0"},{"type":"type","title":"Patch.Listener.t/0","doc":"","ref":"Patch.Listener.html#t:t/0"},{"type":"type","title":"Patch.Listener.tag/0","doc":"Listeners are started with a tag so the listening process can differentiate\nbetween multiple listeners.","ref":"Patch.Listener.html#t:tag/0"},{"type":"type","title":"Patch.Listener.target/0","doc":"Listeners listen to a target.","ref":"Patch.Listener.html#t:target/0"},{"type":"type","title":"Patch.Listener.timeout_option/0","doc":"Option to control how long the listener should wait for GenServer.call\n\nValue is either the number of milliseconds to wait or the `:infinity` atom.\n\nIf `capture_replies` is set to false this setting has no effect.\n\nDefaults to 5000","ref":"Patch.Listener.html#t:timeout_option/0"},{"type":"module","title":"Patch.Listener.Supervisor","doc":"","ref":"Patch.Listener.Supervisor.html"},{"type":"function","title":"Patch.Listener.Supervisor.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Patch.Listener.Supervisor.html#child_spec/1"},{"type":"function","title":"Patch.Listener.Supervisor.init/1","doc":"","ref":"Patch.Listener.Supervisor.html#init/1"},{"type":"function","title":"Patch.Listener.Supervisor.start_child/4","doc":"","ref":"Patch.Listener.Supervisor.html#start_child/4"},{"type":"function","title":"Patch.Listener.Supervisor.start_link/1","doc":"","ref":"Patch.Listener.Supervisor.html#start_link/1"},{"type":"module","title":"Patch.Mock","doc":"","ref":"Patch.Mock.html"},{"type":"function","title":"Patch.Mock.expose/2","doc":"Expose private functions in a module.\n\nIf the module is not already mocked, calling this function will mock it.","ref":"Patch.Mock.html#expose/2"},{"type":"function","title":"Patch.Mock.history/1","doc":"Gets the call history for a module.\n\nIf the module is not already mocked, this function returns an empty new history.","ref":"Patch.Mock.html#history/1"},{"type":"function","title":"Patch.Mock.module/2","doc":"Mocks the given module.\n\nMocking a module accepts two options, see the `t:option/0` type in this module for details.","ref":"Patch.Mock.html#module/2"},{"type":"function","title":"Patch.Mock.register/3","doc":"Registers a mock value for a function.\n\nIf the module is not already mocked, this function will mock it with no private functions\nexposed.","ref":"Patch.Mock.html#register/3"},{"type":"function","title":"Patch.Mock.restore/1","doc":"Restores a module to pre-patch functionality.\n\nIf the module is not already mocked, this function no-ops.","ref":"Patch.Mock.html#restore/1"},{"type":"function","title":"Patch.Mock.restore/2","doc":"Restores a function in a module to pre-patch functionality.\n\nIf the module or function are not already mocked, this function no-ops.","ref":"Patch.Mock.html#restore/2"},{"type":"type","title":"Patch.Mock.exposes/0","doc":"What exposures should be made in a module.\n\n- `:public` will only expose the public functions\n- `:all` will expose both public and private functions\n- A list of exports can be provided, they will be added to the `:public` functions.","ref":"Patch.Mock.html#t:exposes/0"},{"type":"type","title":"Patch.Mock.exposes_option/0","doc":"The exposes option controls if any private functions should be exposed.\n\nThe default is `:public`.","ref":"Patch.Mock.html#t:exposes_option/0"},{"type":"type","title":"Patch.Mock.history_limit_option/0","doc":"This history_limit option controls how large of a history a mock should store\n\nIt defaults to `:infinity` which will store an unlimited history.","ref":"Patch.Mock.html#t:history_limit_option/0"},{"type":"type","title":"Patch.Mock.option/0","doc":"Sum-type of all valid options","ref":"Patch.Mock.html#t:option/0"},{"type":"module","title":"Patch.Mock.History","doc":"","ref":"Patch.Mock.History.html"},{"type":"function","title":"Patch.Mock.History.entries/2","doc":"","ref":"Patch.Mock.History.html#entries/2"},{"type":"function","title":"Patch.Mock.History.new/1","doc":"","ref":"Patch.Mock.History.html#new/1"},{"type":"function","title":"Patch.Mock.History.put/3","doc":"","ref":"Patch.Mock.History.html#put/3"},{"type":"type","title":"Patch.Mock.History.argument/0","doc":"","ref":"Patch.Mock.History.html#t:argument/0"},{"type":"type","title":"Patch.Mock.History.entry/0","doc":"","ref":"Patch.Mock.History.html#t:entry/0"},{"type":"type","title":"Patch.Mock.History.limit/0","doc":"","ref":"Patch.Mock.History.html#t:limit/0"},{"type":"type","title":"Patch.Mock.History.name/0","doc":"","ref":"Patch.Mock.History.html#t:name/0"},{"type":"type","title":"Patch.Mock.History.sorting/0","doc":"","ref":"Patch.Mock.History.html#t:sorting/0"},{"type":"type","title":"Patch.Mock.History.t/0","doc":"","ref":"Patch.Mock.History.html#t:t/0"},{"type":"module","title":"Patch.Mock.History.Tagged","doc":"","ref":"Patch.Mock.History.Tagged.html"},{"type":"function","title":"Patch.Mock.History.Tagged.any?/1","doc":"Determine if any of the entries have been tagged in the affirmative","ref":"Patch.Mock.History.Tagged.html#any?/1"},{"type":"function","title":"Patch.Mock.History.Tagged.count/1","doc":"Calculates the count of entries that have been tagged in the affirmative","ref":"Patch.Mock.History.Tagged.html#count/1"},{"type":"function","title":"Patch.Mock.History.Tagged.first/1","doc":"Returns the first entry that was tagged in the affirmative","ref":"Patch.Mock.History.Tagged.html#first/1"},{"type":"macro","title":"Patch.Mock.History.Tagged.for_call/2","doc":"Construct a new Tagged History from a History and a Call.\n\nEvery entry in the History will be tagged with either `true` if the entry\nmatches the provided call or `false` if the entry does not match.","ref":"Patch.Mock.History.Tagged.html#for_call/2"},{"type":"function","title":"Patch.Mock.History.Tagged.for_function/2","doc":"Construct a new Tagged History from a History and a Function Name.\n\nEvery entry in the History will be tagged with either `true` if the entry\nmatches the provided Function Name or `false` if the entry does not match.","ref":"Patch.Mock.History.Tagged.html#for_function/2"},{"type":"function","title":"Patch.Mock.History.Tagged.format/2","doc":"","ref":"Patch.Mock.History.Tagged.html#format/2"},{"type":"function","title":"Patch.Mock.History.Tagged.tag/1","doc":"Returns the tag for the given tagged entry","ref":"Patch.Mock.History.Tagged.html#tag/1"},{"type":"type","title":"Patch.Mock.History.Tagged.entry/0","doc":"","ref":"Patch.Mock.History.Tagged.html#t:entry/0"},{"type":"type","title":"Patch.Mock.History.Tagged.t/0","doc":"","ref":"Patch.Mock.History.Tagged.html#t:t/0"},{"type":"type","title":"Patch.Mock.History.Tagged.tag/0","doc":"","ref":"Patch.Mock.History.Tagged.html#t:tag/0"},{"type":"module","title":"Patch.Mock.Naming","doc":"","ref":"Patch.Mock.Naming.html"},{"type":"function","title":"Patch.Mock.Naming.delegate/1","doc":"Canonical name for the delegate module for a provided module.","ref":"Patch.Mock.Naming.html#delegate/1"},{"type":"function","title":"Patch.Mock.Naming.facade/1","doc":"Canonical name for the facade module for a provided module.\n\nThe facade module simply takes on the name of the provided module.","ref":"Patch.Mock.Naming.html#facade/1"},{"type":"function","title":"Patch.Mock.Naming.frozen/1","doc":"Canonical name for the frozen module for a provided module.","ref":"Patch.Mock.Naming.html#frozen/1"},{"type":"function","title":"Patch.Mock.Naming.original/1","doc":"Canonical name for the original module for a provided module.","ref":"Patch.Mock.Naming.html#original/1"},{"type":"function","title":"Patch.Mock.Naming.server/1","doc":"Canonical name for the server process for a provided module.","ref":"Patch.Mock.Naming.html#server/1"},{"type":"module","title":"Patch.Mock.Server","doc":"","ref":"Patch.Mock.Server.html"},{"type":"function","title":"Patch.Mock.Server.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Patch.Mock.Server.html#child_spec/1"},{"type":"function","title":"Patch.Mock.Server.delegate/3","doc":"Delegates a call to a mocked module through the server so it can optionally\nrespond with a mock value.","ref":"Patch.Mock.Server.html#delegate/3"},{"type":"function","title":"Patch.Mock.Server.do_restore/1","doc":"","ref":"Patch.Mock.Server.html#do_restore/1"},{"type":"function","title":"Patch.Mock.Server.do_restore/2","doc":"","ref":"Patch.Mock.Server.html#do_restore/2"},{"type":"function","title":"Patch.Mock.Server.expose/2","doc":"","ref":"Patch.Mock.Server.html#expose/2"},{"type":"function","title":"Patch.Mock.Server.history/1","doc":"Retrieves the call history for a mock","ref":"Patch.Mock.Server.html#history/1"},{"type":"function","title":"Patch.Mock.Server.init/1","doc":"","ref":"Patch.Mock.Server.html#init/1"},{"type":"function","title":"Patch.Mock.Server.register/3","doc":"Registers a mock value to be returned whenever the specified function is\ncalled on the module.","ref":"Patch.Mock.Server.html#register/3"},{"type":"function","title":"Patch.Mock.Server.restore/1","doc":"Restores a module to its original state.","ref":"Patch.Mock.Server.html#restore/1"},{"type":"function","title":"Patch.Mock.Server.restore/2","doc":"Restores a function in a module to its original state","ref":"Patch.Mock.Server.html#restore/2"},{"type":"function","title":"Patch.Mock.Server.start_link/2","doc":"","ref":"Patch.Mock.Server.html#start_link/2"},{"type":"type","title":"Patch.Mock.Server.option/0","doc":"Sum-type of all valid options","ref":"Patch.Mock.Server.html#t:option/0"},{"type":"type","title":"Patch.Mock.Server.t/0","doc":"","ref":"Patch.Mock.Server.html#t:t/0"},{"type":"module","title":"Patch.Mock.Supervisor","doc":"","ref":"Patch.Mock.Supervisor.html"},{"type":"function","title":"Patch.Mock.Supervisor.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Patch.Mock.Supervisor.html#child_spec/1"},{"type":"function","title":"Patch.Mock.Supervisor.init/1","doc":"","ref":"Patch.Mock.Supervisor.html#init/1"},{"type":"function","title":"Patch.Mock.Supervisor.start_child/2","doc":"","ref":"Patch.Mock.Supervisor.html#start_child/2"},{"type":"function","title":"Patch.Mock.Supervisor.start_link/1","doc":"","ref":"Patch.Mock.Supervisor.html#start_link/1"},{"type":"module","title":"Patch.Mock.Code","doc":"Patch mocks out modules by generating mock modules and recompiling them for a `target` module.\n\nPatch's approach to mocking a module provides some powerful affordances.\n\n- Private functions can be mocked.\n- Internal function calls are effected by mocks regardless of the function's visibility without\n  having to change the way code is written.\n- Private functions can be optionally exposed in the facade to make it possible to test a\n  private function directly without changing its visibility in code.\n\n# Mocking Strategy\n\nThere are 4 logical modules and 1 GenServer that are involved when mocking a module.\n\nThe 4 logical modules:\n\n- `target` - The module to be mocked.\n- `facade` - The `target` module is replaced by a `facade` module that intercepts all external\n  calls and redirects them to the `delegate` module.\n- `original` - The `target` module is preserved as the `original` module, with the important\n  transformation that all local calls are redirected to the `delegate` module.\n- `delegate` - This module is responsible for checking with the `server` to see if a call is\n  mocked and should be intercepted.  If so, the mock value is returned, otherwise the `original`\n  function is called.\n\nEach `target` module has an associated GenServer, a `Patch.Mock.Server` that has keeps state\nabout the history of function calls and holds the mock data to be returned on interception.  See\n`Patch.Mock.Server` for more information.","ref":"Patch.Mock.Code.html"},{"type":"module","title":"Example Target Module - Patch.Mock.Code","doc":"To better understand how Patch works, consider the following example module.\n\n```elixir\ndefmodule Example do\n  def public_function(argument_1, argument_2) do\n    {:public, private_function(argument_1, argument_2)}\n  end\n\n  defp private_function(argument_1, argument_2) do\n    {:private, argument_1, argument_2}\n  end\nend\n```\n\n### `facade` module\n\nThe `facade` module is automatically generated based off the exports of the `target` module.\nIt takes on the name of the `provided` module.\n\nFor each exported function, a function is generated in the `facade` module that calls the\n`delegate` module.\n\n```elixir\ndefmodule Example do\n  def public_function(argument_1, argument_2) do\n    Patch.Mock.Delegate.For.Example.public_function(argument_1, argument_2)\n  end\nend\n```\n\n### `delegate` module\n\nThe `delegate` module is automatically generated based off all the functions of the `target`\nmodule.  It takes on a name based off the `target` module, see `Patch.Mock.Naming.delegate/1`.\n\nFor each function, a function is generated in the `delegate` module that calls\n`Patch.Mock.Server.delegate/3` delegating to the server named for the `target` module, see\n`Patch.Mock.Naming.server/1`.\n\n```elixir\ndefmodule Patch.Mock.Delegate.For.Example do\n  def public_function(argument_1, argument_2) do\n    Patch.Mock.Server.delegate(\n      Patch.Mock.Server.For.Example,\n      :public_function,\n      [argument_1, argument_2]\n    )\n  end\n\n  def private_function(argument_1, argument_2) do\n    Patch.Mock.Server.delegate(\n      Patch.Mock.Server.For.Example,\n      :private_function,\n      [argument_1, argument_2]\n    )\n  end\nend\n```\n\n### `original` module\n\nThe `original` module takes on a name based off the `provided` module, see\n`Patch.Mock.Naming.original/1`.\n\nThe code is transformed in the following ways.\n  - All local calls are converted into remote calls to the `delegate` module.\n  - All functions are exported\n\n```elixir\ndefmodule Patch.Mock.Original.For.Example do\n  def public_function(argument_1, argument_2) do\n    {:public, Patch.Mock.Delegate.For.Example.private_function(argument_1, argument_2)}\n  end\n\n  def private_function(argument_1, argument_2) do\n    {:private, argument_1, argument_2}\n  end\nend\n```","ref":"Patch.Mock.Code.html#module-example-target-module"},{"type":"module","title":"External Function Calls - Patch.Mock.Code","doc":"First, let's examine how calls from outside the module are treated.","ref":"Patch.Mock.Code.html#module-external-function-calls"},{"type":"module","title":"Public Function Calls - Patch.Mock.Code","doc":"Code calling `Example.public_function/2` has the following call flow.\n\n```text\n[Caller] -> facade -> delegate -> server -> mocked? -> yes   (Intercepted)\n         [Mock Value] <----------------------------|----'\n                                                    -> no -> original   (Run Original Code)\n         [Original Value] <--------------------------------------'\n```\n\nCalling a public function will either return the mocked value if it exists, or fall back to\ncalling the original function.","ref":"Patch.Mock.Code.html#module-public-function-calls"},{"type":"module","title":"Private Function Calls - Patch.Mock.Code","doc":"Code calling `Example.private_function/2` has the following call flow.\n\n```text\n[Caller] --------------------------> facade\n         [UndefinedFunctionError] <-----'\n```\n\nCalling a private function continues to be an error from the external caller's point of view.\n\nThe `expose` option does allow the facade to expose private functions, in those cases the call\nflow just follows the public call flow.","ref":"Patch.Mock.Code.html#module-private-function-calls"},{"type":"module","title":"Internal Calls - Patch.Mock.Code","doc":"Next, let's examine how calls from outside the module are treated.","ref":"Patch.Mock.Code.html#module-internal-calls"},{"type":"module","title":"Public Function Calls - Patch.Mock.Code","doc":"Code in the `original` module calling public functions have their code transformed to call the\n`delegate` module.\n\n```text\noriginal -> delegate -> server -> mocked? -> yes   (Intercepted)\n         [Mock Value] <------------------|----'\n                                          -> no -> original   (Run Original Code)\n         [Original Value] <----------------------------'\n```\n\nSince the call is redirected to the `delegate`, calling a public function will either return the\nmocked value if it exists, or fall back to calling the original function.","ref":"Patch.Mock.Code.html#module-public-function-calls"},{"type":"module","title":"Private Function Call Flow - Patch.Mock.Code","doc":"Code in the `original` module calling private functions have their code transformed to call the\n`delegate` module\n\n```text\noriginal -> delegate -> server -> mocked? -> yes   (Intercepted)\n         [Mock Value] <------------------|----'\n                                          -> no -> original   (Run Original Code)\n         [Original Value] <----------------------------'\n```\n\nSince the call is redirected to the `delegate`, calling a private function will either return the\nmocked value if it exists, or fall back to calling the original function.","ref":"Patch.Mock.Code.html#module-private-function-call-flow"},{"type":"module","title":"Code Generation - Patch.Mock.Code","doc":"For additional details on how Code Generation works, see the `Patch.Mock.Code.Generate` module.","ref":"Patch.Mock.Code.html#module-code-generation"},{"type":"function","title":"Patch.Mock.Code.abstract_forms/1","doc":"Extracts the abstract_forms from a module","ref":"Patch.Mock.Code.html#abstract_forms/1"},{"type":"function","title":"Patch.Mock.Code.attributes/1","doc":"Extracts the attribtues from a module","ref":"Patch.Mock.Code.html#attributes/1"},{"type":"function","title":"Patch.Mock.Code.classify_export/3","doc":"Classifies an exported mfa into one of the following classifications\n\n- :builtin - Export is a BIF.\n- :generated - Export is a generated function.\n- :normal - Export is a user defined function.","ref":"Patch.Mock.Code.html#classify_export/3"},{"type":"function","title":"Patch.Mock.Code.compile/2","doc":"Compiles the provided abstract_form with the given compiler_options\n\nIn addition to compiling, the module will be loaded.","ref":"Patch.Mock.Code.html#compile/2"},{"type":"function","title":"Patch.Mock.Code.compiler_options/1","doc":"Extracts the compiler options from a module.","ref":"Patch.Mock.Code.html#compiler_options/1"},{"type":"function","title":"Patch.Mock.Code.exports/3","doc":"Extracts the exports from the provided abstract_forms for the module.\n\nThe exports returned can be controlled by the exposes argument.","ref":"Patch.Mock.Code.html#exports/3"},{"type":"function","title":"Patch.Mock.Code.filter_exports/3","doc":"Given a module and a list of exports filters the list of exports to those that\nhave the given classification.\n\nSee `classify_export/3` for information about export classification","ref":"Patch.Mock.Code.html#filter_exports/3"},{"type":"function","title":"Patch.Mock.Code.freeze/1","doc":"Freezes a module by generating a copy of it under a frozen name with all remote calls to the\n`target` module re-routed to the frozen module.","ref":"Patch.Mock.Code.html#freeze/1"},{"type":"function","title":"Patch.Mock.Code.module/2","doc":"Mocks a module by generating a set of modules based on the `target` module.\n\nThe `target` module's Unit is returned on success.","ref":"Patch.Mock.Code.html#module/2"},{"type":"function","title":"Patch.Mock.Code.purge/1","doc":"Purges a module from the code server","ref":"Patch.Mock.Code.html#purge/1"},{"type":"function","title":"Patch.Mock.Code.stick_module/1","doc":"Marks a module a sticky","ref":"Patch.Mock.Code.html#stick_module/1"},{"type":"function","title":"Patch.Mock.Code.unstick_module/1","doc":"Unsticks a module\n\nReturns `{:ok, was_sticky?}` on success, `{:error, reason}` otherwise","ref":"Patch.Mock.Code.html#unstick_module/1"},{"type":"type","title":"Patch.Mock.Code.binary_error/0","doc":"","ref":"Patch.Mock.Code.html#t:binary_error/0"},{"type":"type","title":"Patch.Mock.Code.chunk_error/0","doc":"","ref":"Patch.Mock.Code.html#t:chunk_error/0"},{"type":"type","title":"Patch.Mock.Code.compiler_option/0","doc":"","ref":"Patch.Mock.Code.html#t:compiler_option/0"},{"type":"type","title":"Patch.Mock.Code.export_classification/0","doc":"","ref":"Patch.Mock.Code.html#t:export_classification/0"},{"type":"type","title":"Patch.Mock.Code.exports/0","doc":"","ref":"Patch.Mock.Code.html#t:exports/0"},{"type":"type","title":"Patch.Mock.Code.form/0","doc":"","ref":"Patch.Mock.Code.html#t:form/0"},{"type":"type","title":"Patch.Mock.Code.load_error/0","doc":"","ref":"Patch.Mock.Code.html#t:load_error/0"},{"type":"type","title":"Patch.Mock.Code.option/0","doc":"Sum-type of all valid options","ref":"Patch.Mock.Code.html#t:option/0"},{"type":"module","title":"Patch.Mock.Code.Freezer","doc":"The Code Freezer is a registry that can be used to register and use modules\nthat might be frozen.\n\nModules that Patch relies on must be freezable so that the end-user can Patch\nthem and the frozen versions are still available for internal use.","ref":"Patch.Mock.Code.Freezer.html"},{"type":"function","title":"Patch.Mock.Code.Freezer.empty/0","doc":"Destroy all frozen modules","ref":"Patch.Mock.Code.Freezer.html#empty/0"},{"type":"function","title":"Patch.Mock.Code.Freezer.get/1","doc":"Get the possibly-frozen module to use for a module.\n\nIf the module is frozen then the frozen name will be returned.\n\nIf the module is not frozen then the module is returned.s","ref":"Patch.Mock.Code.Freezer.html#get/1"},{"type":"function","title":"Patch.Mock.Code.Freezer.put/1","doc":"Puts a module into the freezer.\n\nThe module must be freezable.  Repeated calls for frozen modules are no-ops.","ref":"Patch.Mock.Code.Freezer.html#put/1"},{"type":"module","title":"Patch.Mock.Code.Generate","doc":"This module provides the ability to generate derivative modules based on a target module.\n\nThis module delegates code generation to purpose specific Generator modules, see those modules\nfor additional details on how specific generation works.\n\n- `Patch.Mock.Code.Generators.Delegate` for how `delegate` modules are generated\n- `Patch.Mock.Code.Generators.Facade` for how `facade` modules are generated\n- `Patch.Mock.Code.Generators.Frozen` for how `frozen` modules are generated\n- `Patch.Mock.Code.Generators.Original` for how `original` modules are generated","ref":"Patch.Mock.Code.Generate.html"},{"type":"function","title":"Patch.Mock.Code.Generate.delegate/3","doc":"","ref":"Patch.Mock.Code.Generate.html#delegate/3"},{"type":"function","title":"Patch.Mock.Code.Generate.facade/3","doc":"","ref":"Patch.Mock.Code.Generate.html#facade/3"},{"type":"function","title":"Patch.Mock.Code.Generate.frozen/2","doc":"","ref":"Patch.Mock.Code.Generate.html#frozen/2"},{"type":"function","title":"Patch.Mock.Code.Generate.original/3","doc":"","ref":"Patch.Mock.Code.Generate.html#original/3"},{"type":"module","title":"Patch.Mock.Code.Query","doc":"This module provides the ability to query abstract forms.\n\nThis module delegates queries to purpose specific Queries modules, see those modules for\nadditional details on how specific queries work.\n\n- `Patch.Mock.Code.Queries.Exports` for how exports are queried\n- `Patch.Mock.Code.Queries.Functions` for how functions are queried","ref":"Patch.Mock.Code.Query.html"},{"type":"function","title":"Patch.Mock.Code.Query.exports/1","doc":"","ref":"Patch.Mock.Code.Query.html#exports/1"},{"type":"function","title":"Patch.Mock.Code.Query.functions/1","doc":"","ref":"Patch.Mock.Code.Query.html#functions/1"},{"type":"module","title":"Patch.Mock.Code.Transform","doc":"This module provides the ability to transform abstract forms.\n\nThis module delegates transformation to purpose specific Transforms modules, see those modules\nfor additional details on how specific transformations work.\n\n- `Patch.Mock.Code.Transforms.Clean` for how a module is prepared for generation.\n- `Patch.Mock.Code.Transforms.Export` for how exports are rewritten\n- `Patch.Mock.Code.Transforms.Filter` for how functions are filtered\n- `Patch.Mock.Code.Transforms.Remote` for how local calls are made remote\n- `Patch.Mock.Code.Transforms.Rename` for how a module is renamed\n- `Patch.Mock.Code.Transforms.Reroute` for how remote calls are rerouted","ref":"Patch.Mock.Code.Transform.html"},{"type":"function","title":"Patch.Mock.Code.Transform.clean/1","doc":"","ref":"Patch.Mock.Code.Transform.html#clean/1"},{"type":"function","title":"Patch.Mock.Code.Transform.export/2","doc":"","ref":"Patch.Mock.Code.Transform.html#export/2"},{"type":"function","title":"Patch.Mock.Code.Transform.filter/2","doc":"","ref":"Patch.Mock.Code.Transform.html#filter/2"},{"type":"function","title":"Patch.Mock.Code.Transform.remote/2","doc":"","ref":"Patch.Mock.Code.Transform.html#remote/2"},{"type":"function","title":"Patch.Mock.Code.Transform.rename/2","doc":"","ref":"Patch.Mock.Code.Transform.html#rename/2"},{"type":"function","title":"Patch.Mock.Code.Transform.reroute/3","doc":"","ref":"Patch.Mock.Code.Transform.html#reroute/3"},{"type":"module","title":"Patch.Mock.Code.Unit","doc":"","ref":"Patch.Mock.Code.Unit.html"},{"type":"function","title":"Patch.Mock.Code.Unit.purge/1","doc":"","ref":"Patch.Mock.Code.Unit.html#purge/1"},{"type":"function","title":"Patch.Mock.Code.Unit.restore/1","doc":"","ref":"Patch.Mock.Code.Unit.html#restore/1"},{"type":"type","title":"Patch.Mock.Code.Unit.t/0","doc":"","ref":"Patch.Mock.Code.Unit.html#t:t/0"},{"type":"module","title":"Patch.Mock.Code.Generators.Delegate","doc":"Generator for `delegate` modules.\n\n`delegate` modules are generated by taking the `target` module and creating a  stub function for\neach function in the module that calls the `Patch.Mock.Server`'s `delegate/3` function.\n\nThe `delegate` module will also expose every function in the module regardless of the original\nvisibility.","ref":"Patch.Mock.Code.Generators.Delegate.html"},{"type":"function","title":"Patch.Mock.Code.Generators.Delegate.generate/3","doc":"Generates a new delegate module based on the forms of a provided module.","ref":"Patch.Mock.Code.Generators.Delegate.html#generate/3"},{"type":"module","title":"Patch.Mock.Code.Generators.Facade","doc":"Generator for `facade` modules.\n\n`facade` modules are generated by taking the `target` module and creating a stub function for\neach function in the module that calls the `delegate` modules corresponding function.\n\nThe `facade` module can optionally expose private functions as public.","ref":"Patch.Mock.Code.Generators.Facade.html"},{"type":"function","title":"Patch.Mock.Code.Generators.Facade.generate/3","doc":"Generates a new facade module based on the forms of the provided module.","ref":"Patch.Mock.Code.Generators.Facade.html#generate/3"},{"type":"module","title":"Patch.Mock.Code.Generators.Frozen","doc":"Generator for `frozen` modules.\n\n`frozen` modules are generated by taking the `target` module and renaming it and ensuring\nthat any call in the module to itself is sent to the `frozen` module and not the provided\nmodule.\n\nThe `frozen` module simple acts as a way to call the unaltered original code.  It is only\nused internally to preserve the GenServer module since the internal implementation requires that\nGenServer work as expected.","ref":"Patch.Mock.Code.Generators.Frozen.html"},{"type":"function","title":"Patch.Mock.Code.Generators.Frozen.generate/2","doc":"Generates a new frozen module based on the forms of a provided module.","ref":"Patch.Mock.Code.Generators.Frozen.html#generate/2"},{"type":"module","title":"Patch.Mock.Code.Generators.Original","doc":"","ref":"Patch.Mock.Code.Generators.Original.html"},{"type":"function","title":"Patch.Mock.Code.Generators.Original.generate/3","doc":"Generates a new original module based on the forms of the provided module.","ref":"Patch.Mock.Code.Generators.Original.html#generate/3"},{"type":"module","title":"Patch.Mock.Code.Queries.Exports","doc":"","ref":"Patch.Mock.Code.Queries.Exports.html"},{"type":"function","title":"Patch.Mock.Code.Queries.Exports.query/1","doc":"Queries the provided forms for the exported functions.","ref":"Patch.Mock.Code.Queries.Exports.html#query/1"},{"type":"module","title":"Patch.Mock.Code.Queries.Functions","doc":"","ref":"Patch.Mock.Code.Queries.Functions.html"},{"type":"function","title":"Patch.Mock.Code.Queries.Functions.query/1","doc":"Queries the provided forms for all defined functions.","ref":"Patch.Mock.Code.Queries.Functions.html#query/1"},{"type":"module","title":"Patch.Mock.Code.Transforms.Clean","doc":"Cleans abstract form to prepare it for subsequent generation.\n\nThe following forms are retained:\n  - module attribute\n  - exports\n  - no_auto_import compiler options (This is needed to prevent R14 shadow import warnings)\n  - functions","ref":"Patch.Mock.Code.Transforms.Clean.html"},{"type":"function","title":"Patch.Mock.Code.Transforms.Clean.transform/1","doc":"","ref":"Patch.Mock.Code.Transforms.Clean.html#transform/1"},{"type":"module","title":"Patch.Mock.Code.Transforms.Export","doc":"","ref":"Patch.Mock.Code.Transforms.Export.html"},{"type":"function","title":"Patch.Mock.Code.Transforms.Export.transform/2","doc":"Transforms the provided forms to export the given list of functions.","ref":"Patch.Mock.Code.Transforms.Export.html#transform/2"},{"type":"module","title":"Patch.Mock.Code.Transforms.Filter","doc":"Filters the functions and specifications in a module to the exports provided.","ref":"Patch.Mock.Code.Transforms.Filter.html"},{"type":"function","title":"Patch.Mock.Code.Transforms.Filter.transform/2","doc":"","ref":"Patch.Mock.Code.Transforms.Filter.html#transform/2"},{"type":"module","title":"Patch.Mock.Code.Transforms.Remote","doc":"","ref":"Patch.Mock.Code.Transforms.Remote.html"},{"type":"function","title":"Patch.Mock.Code.Transforms.Remote.transform/2","doc":"Transforms the provided forms to rewrite any local call into a remote call to\nthe provided `module`.","ref":"Patch.Mock.Code.Transforms.Remote.html#transform/2"},{"type":"module","title":"Patch.Mock.Code.Transforms.Rename","doc":"","ref":"Patch.Mock.Code.Transforms.Rename.html"},{"type":"function","title":"Patch.Mock.Code.Transforms.Rename.transform/2","doc":"Transforms the provided forms to rename the module to the provided module\nname.","ref":"Patch.Mock.Code.Transforms.Rename.html#transform/2"},{"type":"module","title":"Patch.Mock.Code.Transforms.Reroute","doc":"","ref":"Patch.Mock.Code.Transforms.Reroute.html"},{"type":"function","title":"Patch.Mock.Code.Transforms.Reroute.transform/3","doc":"Transforms the provided forms to rewrite any remote calls to the `source` module into remote\ncalls to the `destination` module.","ref":"Patch.Mock.Code.Transforms.Reroute.html#transform/3"},{"type":"module","title":"Patch.Mock.Value","doc":"Interface for generating mock values.\n\nIn a test this module is imported into the test and so using this module directly is not\nnecessary.","ref":"Patch.Mock.Value.html"},{"type":"function","title":"Patch.Mock.Value.advance/1","doc":"Advances the given value.\n\nSequences and Cycles both have meaningful advances, all other values types this acts as a no-op.","ref":"Patch.Mock.Value.html#advance/1"},{"type":"function","title":"Patch.Mock.Value.callable/1","doc":"Create a new `Values.Callable` to be used as the mock value.\n\nWhen a patched function has a `Values.Callable` as its mock value, it will invoke the callable\nwith the arguments to the patched function on every invocation to generate a new value to\nreturn.\n\n```elixir\npatch(Example, :example, callable(fn arg -> {:patched, arg} end))\n\nassert Example.example(1) == {:patched, 1}   # passes\nassert Example.example(2) == {:patched, 2}   # passes\nassert Example.example(3) == {:patched, 3}   # passes\n```\n\nAny function literal will automatically be promoted into a `Values.Callable` unless it is\nwrapped in a `scalar/1` call.\n\nSee `callable/2` for more configuration options.  `callable/1` calls use the default\nconfiguration options\n\n- dispatch: :apply\n- evaluate: :passthrough","ref":"Patch.Mock.Value.html#callable/1"},{"type":"function","title":"Patch.Mock.Value.callable/2","doc":"`callable/2` allows the test author to provide additional configuration.\n\nThere are two options\n\n## `:dispatch`\n\nControls how the arguments are dispatched to the callable.\n\n- `:apply` is the default.  It will call the function with the same arity as the incoming call.\n- `:list` will always call the callable with a single argument, a list of all the incoming\n  arguments.","ref":"Patch.Mock.Value.html#callable/2"},{"type":"function","title":"Apply Example - Patch.Mock.Value.callable/2","doc":"```elixir\npatch(Example, :example, callable(fn a, b, c -> {:patched, a, b, c} end), :apply)\n\nassert Example.example(1, 2, 3)  == {:patched, 1, 2, 3}   # passes\n```","ref":"Patch.Mock.Value.html#callable/2-apply-example"},{"type":"function","title":"List Example - Patch.Mock.Value.callable/2","doc":"```elixir\npatch(Example, :example, callable(fn\n  [a, b, c] ->\n    {:patched, a, b, c}\n\n  [a] ->\n    {:patched, a}\nend, :list))\n\nassert Example.example(1, 2, 3) == {:patched, 1, 2, 3}   # passes\nassert Example.example(1) == {:patched, 1}   # passes\n```\n\n## `:evaluate`\n\nControls how the callable is evaluated.\n\n- `:passthrough` is the default.  It will passthrough to the original function if the provided\n  callable fails to pattern match to the incoming call\n- `:strict` will bubble up any `BadArityError` or `FunctionClauseErrors`.","ref":"Patch.Mock.Value.html#callable/2-list-example"},{"type":"function","title":"Legacy Configuration Behavior (may be deprecated) - Patch.Mock.Value.callable/2","doc":"This function accepts either a single atom, in which case it will assign that to the `:dispatch`\nconfiguration and use the default `:evaluate` option.\n\nThe following calls are equivalent\n\n```elixir\n# Using legacy configuration convention\npatch(Example, :example, callable(fn args -> {:patched, args}, :apply))\n\n# Using explicit options without evaluate\npatch(Example, :example, callable(fn args -> {:patched, args}, dispatch: :apply))\n\n# Using fully specified explicit options\npatch(Example, :example, callable(fn args -> {:patched, args}, dispatch: :apply, evaluate: :passthrough))\n```","ref":"Patch.Mock.Value.html#callable/2-legacy-configuration-behavior-may-be-deprecated"},{"type":"function","title":"Multiple Arities - Patch.Mock.Value.callable/2","doc":"`dispatch: :list` used to be the preferred way to deal with multiple arities, here's an example.\n\n```elixir\npatch(Example, :example, callable(fn\n  [a] ->\n    {:patched, a}\n\n  [a, b, c] ->\n    {:patched, a, b, c}\nend, dispatch: :list))\n\nassert Example.example(1) == {:patched, 1}\nassert Example.example(1, 2, 3) == {:patched, 1, 2, 3}\n```\n\nPatch now has \"Stacked Callables\" so the preferred method is to use the equivalent code\n\n```elixir\npatch(Example, :example, fn a -> {:patched, a} end)\npatch(Example, :example, fn a, b, c -> {:patched, a, b, c} end)\n\nassert Example.example(1) == {:patched, 1}\nassert Example.example(1, 2, 3) == {:patched, 1, 2, 3}\n```","ref":"Patch.Mock.Value.html#callable/2-multiple-arities"},{"type":"function","title":"Patch.Mock.Value.cycle/1","doc":"Create a new `Values.Cycle` to be used as the mock value.\n\nWhen a patched function has a `Values.Cycle` as its mock value, it will provide the first value\nin the cycle and then move the first value to the end of the cycle on every invocation.\n\nConsider a function patched with `cycle([1, 2, 3])` via the following code\n\n```elixir\npatch(Example, :example, cycle([1, 2, 3]))\n```\n\n| Invocation | Cycle Before Call | Return Value | Cycle After Call |\n|------------|-------------------|--------------|------------------|\n| 1          | [1, 2, 3]         | 1            | [2, 3, 1]        |\n| 2          | [2, 3, 1]         | 2            | [3, 1, 2]        |\n| 3          | [3, 1, 2]         | 3            | [1, 2, 3]        |\n| 4          | [1, 2, 3]         | 1            | [2, 3, 1]        |\n| 5          | [2, 3, 1]         | 2            | [3, 1, 2]        |\n| 6          | [3, 1, 2]         | 3            | [1, 2, 3]        |\n| 7          | [1, 2, 3]         | 1            | [2, 3, 1]        |\n\nWe could continue the above table forever since the cycle will repeat endlessly.  Cycles can\ncontain `callable/1,2`, `raise/1,2` and `throw/1` mock values.","ref":"Patch.Mock.Value.html#cycle/1"},{"type":"macro","title":"Patch.Mock.Value.is_value/1","doc":"Guard that checks whether a value is a proper Values module","ref":"Patch.Mock.Value.html#is_value/1"},{"type":"function","title":"Patch.Mock.Value.next/2","doc":"Generate the next return value and advance the underlying value.","ref":"Patch.Mock.Value.html#next/2"},{"type":"function","title":"Patch.Mock.Value.raises/1","doc":"Creates a special value that raises a RuntimeError with the given message.\n\n```elixir\npatch(Example, :example, raises(\"patched\"))\n\nassert_raise RuntimeError, \"patched\", fn ->\n  Example.example()\nend\n```","ref":"Patch.Mock.Value.html#raises/1"},{"type":"function","title":"Patch.Mock.Value.raises/2","doc":"Creates a special value that raises the given exception with the provided attributes.\n\n```elixir\npatch(Example, :example, raises(ArgumentError, message: \"patched\"))\n\nassert_raise ArgumentError, \"patched\", fn ->\n  Example.example()\nend\n```","ref":"Patch.Mock.Value.html#raises/2"},{"type":"function","title":"Patch.Mock.Value.scalar/1","doc":"Creates a new `Values.Scalar` to be used as the mock value.\n\nWhen a patched function has a `Values.Scalar` as its mock value, it will provide the scalar\nvalue on every invocation\n\n```elixir\npatch(Example, :example, scalar(:patched))\n\nassert Example.example() == :patched   # passes\nassert Example.example() == :patched   # passes\nassert Example.example() == :patched   # passes\n```\n\nWhen patching with any term that isn't a function, it will automatically be promoted into a\n`Values.Scalar`.\n\n```elixir\npatch(Example, :example, :patched)\n\nassert Example.example() == :patched   # passes\nassert Example.example() == :patched   # passes\nassert Example.example() == :patched   # passes\n```\n\nSince functions are always automatically promoted to `Values.Callable`, if a function is meant\nas a scalar value it **must** be wrapped in a call to `scalar/1`.\n\n```elixir\npatch(Example, :get_name_normalizer, scalar(&String.downcase/1))\n\nassert Example.get_name_normalizer == &String.downcase/1   # passes\n```","ref":"Patch.Mock.Value.html#scalar/1"},{"type":"function","title":"Patch.Mock.Value.sequence/1","doc":"Creates a new `Values.Sequence` to be used as a mock value.\n\nWhen a patched function has a `Values.Sequence` as its mock value, it will provide the first\nvalue in the sequence as the return value and then discard the first value.  Once the sequence\nis down to a final value it will be retained and returned on every subsequent invocation.\n\nConsider a function patched with `sequence([1, 2, 3])` via the following code\n\n```elixir\npatch(Example, :example, sequence([1, 2, 3]))\n```\n\n| Invocation | Sequence Before Call | Return Value | Sequence After Call |\n|------------|----------------------|--------------|---------------------|\n| 1          | [1, 2, 3]            | 1            | [2, 3]              |\n| 2          | [2, 3]               | 2            | [3]                 |\n| 3          | [3]                  | 3            | [3]                 |\n| 4          | [3]                  | 3            | [3]                 |\n| 5          | [3]                  | 3            | [3]                 |\n\nWe could continue the above table forever since the sequence will continue to return the last\nvalue endlessly.  Sequences can contain `callable/1,2`, `raise/1,2` and `throw/1` mock values.\n\nThere is one special behavior of sequence, and that's an empty sequence, which always returns\nthe value `nil` on every invocation.\n\nIf the test author would like to simulate an exhaustable sequence, one that returns a set number\nof items and then responds to every other call with `nil`, they can simply add a `nil` as the\nlast element in the sequence\n\n```elixir\npatch(Example, :example, sequence([1, 2, 3, nil])\n```\n\n| Invocation | Sequence Before Call | Return Value | Sequence After Call |\n|------------|----------------------|--------------|---------------------|\n| 1          | [1, 2, 3, nil]       | 1            | [2, 3, nil]         |\n| 2          | [2, 3, nil]          | 2            | [3, nil]            |\n| 3          | [3, nil]             | 3            | [nil]               |\n| 4          | [nil]                | nil          | [nil]               |\n| 5          | [nil]                | nil          | [nil]               |","ref":"Patch.Mock.Value.html#sequence/1"},{"type":"function","title":"Patch.Mock.Value.throws/1","doc":"Creates a special values that throws the provided value when evaluated.\n\n```elixir\npatch(Example, :example, throws(:patched))\n\nassert catch_throw(Example.example()) == :patched\n```","ref":"Patch.Mock.Value.html#throws/1"},{"type":"type","title":"Patch.Mock.Value.t/0","doc":"","ref":"Patch.Mock.Value.html#t:t/0"},{"type":"module","title":"Patch.Mock.Values.Callable","doc":"","ref":"Patch.Mock.Values.Callable.html"},{"type":"function","title":"Patch.Mock.Values.Callable.advance/1","doc":"","ref":"Patch.Mock.Values.Callable.html#advance/1"},{"type":"function","title":"Patch.Mock.Values.Callable.new/2","doc":"","ref":"Patch.Mock.Values.Callable.html#new/2"},{"type":"function","title":"Patch.Mock.Values.Callable.next/2","doc":"","ref":"Patch.Mock.Values.Callable.html#next/2"},{"type":"type","title":"Patch.Mock.Values.Callable.dispatch_mode/0","doc":"","ref":"Patch.Mock.Values.Callable.html#t:dispatch_mode/0"},{"type":"type","title":"Patch.Mock.Values.Callable.dispatch_option/0","doc":"","ref":"Patch.Mock.Values.Callable.html#t:dispatch_option/0"},{"type":"type","title":"Patch.Mock.Values.Callable.evaluate_mode/0","doc":"","ref":"Patch.Mock.Values.Callable.html#t:evaluate_mode/0"},{"type":"type","title":"Patch.Mock.Values.Callable.evaluate_option/0","doc":"","ref":"Patch.Mock.Values.Callable.html#t:evaluate_option/0"},{"type":"type","title":"Patch.Mock.Values.Callable.option/0","doc":"","ref":"Patch.Mock.Values.Callable.html#t:option/0"},{"type":"type","title":"Patch.Mock.Values.Callable.t/0","doc":"","ref":"Patch.Mock.Values.Callable.html#t:t/0"},{"type":"module","title":"Patch.Mock.Values.CallableStack","doc":"","ref":"Patch.Mock.Values.CallableStack.html"},{"type":"function","title":"Patch.Mock.Values.CallableStack.advance/1","doc":"","ref":"Patch.Mock.Values.CallableStack.html#advance/1"},{"type":"function","title":"Patch.Mock.Values.CallableStack.new/1","doc":"","ref":"Patch.Mock.Values.CallableStack.html#new/1"},{"type":"function","title":"Patch.Mock.Values.CallableStack.next/2","doc":"","ref":"Patch.Mock.Values.CallableStack.html#next/2"},{"type":"function","title":"Patch.Mock.Values.CallableStack.push/2","doc":"","ref":"Patch.Mock.Values.CallableStack.html#push/2"},{"type":"type","title":"Patch.Mock.Values.CallableStack.t/0","doc":"","ref":"Patch.Mock.Values.CallableStack.html#t:t/0"},{"type":"module","title":"Patch.Mock.Values.Cycle","doc":"","ref":"Patch.Mock.Values.Cycle.html"},{"type":"function","title":"Patch.Mock.Values.Cycle.advance/1","doc":"","ref":"Patch.Mock.Values.Cycle.html#advance/1"},{"type":"function","title":"Patch.Mock.Values.Cycle.new/1","doc":"","ref":"Patch.Mock.Values.Cycle.html#new/1"},{"type":"function","title":"Patch.Mock.Values.Cycle.next/2","doc":"","ref":"Patch.Mock.Values.Cycle.html#next/2"},{"type":"type","title":"Patch.Mock.Values.Cycle.t/0","doc":"","ref":"Patch.Mock.Values.Cycle.html#t:t/0"},{"type":"module","title":"Patch.Mock.Values.Raises","doc":"","ref":"Patch.Mock.Values.Raises.html"},{"type":"function","title":"Patch.Mock.Values.Raises.advance/1","doc":"","ref":"Patch.Mock.Values.Raises.html#advance/1"},{"type":"function","title":"Patch.Mock.Values.Raises.new/1","doc":"","ref":"Patch.Mock.Values.Raises.html#new/1"},{"type":"function","title":"Patch.Mock.Values.Raises.new/2","doc":"","ref":"Patch.Mock.Values.Raises.html#new/2"},{"type":"function","title":"Patch.Mock.Values.Raises.next/2","doc":"","ref":"Patch.Mock.Values.Raises.html#next/2"},{"type":"type","title":"Patch.Mock.Values.Raises.t/0","doc":"","ref":"Patch.Mock.Values.Raises.html#t:t/0"},{"type":"module","title":"Patch.Mock.Values.Scalar","doc":"","ref":"Patch.Mock.Values.Scalar.html"},{"type":"function","title":"Patch.Mock.Values.Scalar.advance/1","doc":"","ref":"Patch.Mock.Values.Scalar.html#advance/1"},{"type":"function","title":"Patch.Mock.Values.Scalar.new/1","doc":"","ref":"Patch.Mock.Values.Scalar.html#new/1"},{"type":"function","title":"Patch.Mock.Values.Scalar.next/2","doc":"","ref":"Patch.Mock.Values.Scalar.html#next/2"},{"type":"type","title":"Patch.Mock.Values.Scalar.t/0","doc":"","ref":"Patch.Mock.Values.Scalar.html#t:t/0"},{"type":"module","title":"Patch.Mock.Values.Sequence","doc":"","ref":"Patch.Mock.Values.Sequence.html"},{"type":"function","title":"Patch.Mock.Values.Sequence.advance/1","doc":"","ref":"Patch.Mock.Values.Sequence.html#advance/1"},{"type":"function","title":"Patch.Mock.Values.Sequence.new/1","doc":"","ref":"Patch.Mock.Values.Sequence.html#new/1"},{"type":"function","title":"Patch.Mock.Values.Sequence.next/2","doc":"","ref":"Patch.Mock.Values.Sequence.html#next/2"},{"type":"type","title":"Patch.Mock.Values.Sequence.t/0","doc":"","ref":"Patch.Mock.Values.Sequence.html#t:t/0"},{"type":"module","title":"Patch.Mock.Values.Throws","doc":"","ref":"Patch.Mock.Values.Throws.html"},{"type":"function","title":"Patch.Mock.Values.Throws.advance/1","doc":"","ref":"Patch.Mock.Values.Throws.html#advance/1"},{"type":"function","title":"Patch.Mock.Values.Throws.new/1","doc":"","ref":"Patch.Mock.Values.Throws.html#new/1"},{"type":"function","title":"Patch.Mock.Values.Throws.next/2","doc":"","ref":"Patch.Mock.Values.Throws.html#next/2"},{"type":"type","title":"Patch.Mock.Values.Throws.t/0","doc":"","ref":"Patch.Mock.Values.Throws.html#t:t/0"},{"type":"module","title":"Patch.Access","doc":"","ref":"Patch.Access.html"},{"type":"function","title":"Patch.Access.fetch/2","doc":"","ref":"Patch.Access.html#fetch/2"},{"type":"function","title":"Patch.Access.get/3","doc":"","ref":"Patch.Access.html#get/3"},{"type":"function","title":"Patch.Access.put/3","doc":"","ref":"Patch.Access.html#put/3"},{"type":"module","title":"Patch.Apply","doc":"Utility module to assist with applying functions.","ref":"Patch.Apply.html"},{"type":"function","title":"Patch.Apply.safe/2","doc":"Safe application of an anonymous function.\n\nThis is just like `apply/2` but if the the anonymous function is unable to\nhandle the call (raising either `BadArityError` or `FunctionClauseError`) then\nthe `:error` atom is returned.\n\nIf the function evaluates successfully its reply is returned wrapped in an\n`{:ok, result}` tuple.","ref":"Patch.Apply.html#safe/2"},{"type":"module","title":"Patch.Assertions","doc":"","ref":"Patch.Assertions.html"},{"type":"function","title":"Patch.Assertions.assert_any_call/2","doc":"Asserts that the given module and function has been called with any arity.\n\n```elixir\npatch(Example, :function, :patch)\n\nPatch.Assertions.assert_any_call(Example, :function)   # fails\n\nExample.function(1, 2, 3)\n\nPatch.Asertions.assert_any_call(Example, :function)   # passes\n```\n\nThere are convenience delegates in the Developer Interface, `Patch.assert_any_call/1` and\n`Patch.assert_any_call/2` which should be preferred over calling this function directly.","ref":"Patch.Assertions.html#assert_any_call/2"},{"type":"macro","title":"Patch.Assertions.assert_called/1","doc":"Given a call will assert that a matching call was observed by the patched function.\n\nThis macro fully supports patterns and will perform non-hygienic binding similar to ExUnit's\n`assert_receive/3` and `assert_received/2`.\n\n```elixir\npatch(Example, :function, :patch)\n\nExample.function(1, 2, 3)\n\nPatch.Assertions.assert_called(Example, :function, [1, 2, 3])   # passes\nPatch.Assertions.assert_called(Example, :function, [1, _, 3])   # passes\nPatch.Assertions.assert_called(Example, :function, [4, 5, 6])   # fails\nPatch.Assertions.assert_called(Example, :function, [4, _, 6])   # fails\n```\n\nThere is a convenience macro in the Developer Interface, `Patch.assert_called/1` which should be\npreferred over calling this macro directly.","ref":"Patch.Assertions.html#assert_called/1"},{"type":"macro","title":"Patch.Assertions.assert_called/2","doc":"Given a call will assert that a matching call was observed exactly the number of times provided\nby the patched function.\n\nThis macro fully supports patterns and will perform non-hygienic binding similar to ExUnit's\n`assert_receive/3` and `assert_received/2`.  The value bound will be the from the latest call.\n\n```elixir\npatch(Example, :function, :patch)\n\nExample.function(1, 2, 3)\n\nPatch.Assertions.assert_called(Example, :function, [1, 2, 3], 1)   # passes\nPatch.Assertions.assert_called(Example, :function, [1, _, 3], 1)  # passes\n\nExample.function(1, 2, 3)\n\nPatch.Assertions.assert_called(Example, :function, [1, 2, 3], 2)   # passes\nPatch.Assertions.assert_called(Example, :function, [1, _, 3], 2)  # passes\n```\n\nThere is a convenience macro in the Developer Interface, `Patch.assert_called/2` which\nshould be preferred over calling this macro directly.","ref":"Patch.Assertions.html#assert_called/2"},{"type":"macro","title":"Patch.Assertions.assert_called_once/1","doc":"Given a call will assert that a matching call was observed exactly once by the patched function.\n\nThis macro fully supports patterns and will perform non-hygienic binding similar to ExUnit's\n`assert_receive/3` and `assert_received/2`.\n\n```elixir\npatch(Example, :function, :patch)\n\nExample.function(1, 2, 3)\n\nPatch.Assertions.assert_called_once(Example, :function, [1, 2, 3])   # passes\nPatch.Assertions.assert_called_once(Example, :function, [1, _, 3])  # passes\n\nExample.function(1, 2, 3)\n\nPatch.Assertions.assert_called_once(Example, :function, [1, 2, 3])   # fails\nPatch.Assertions.assert_called_once(Example, :function, [1, _, 3])  # fails\n```\n\nThere is a convenience macro in the Developer Interface, `Patch.assert_called_once/1` which\nshould be preferred over calling this macro directly.","ref":"Patch.Assertions.html#assert_called_once/1"},{"type":"macro","title":"Patch.Assertions.format_patterns/1","doc":"Formats the AST for a list of patterns AST as they would appear in an argument list.","ref":"Patch.Assertions.html#format_patterns/1"},{"type":"function","title":"Patch.Assertions.refute_any_call/2","doc":"Refutes that the given module and function has been called with any arity.\n\n```elixir\npatch(Example, :function, :patch)\n\nPatch.Assertions.refute_any_call(Example, :function)   # passes\n\nExample.function(1, 2, 3)\n\nPatch.Assertions.refute_any_call(Example, :function)   # fails\n```\n\nThere are convenience delegates in the Developer Interface, `Patch.refute_any_call/1` and\n`Patch.refute_any_call/2` which should be preferred over calling this function directly.","ref":"Patch.Assertions.html#refute_any_call/2"},{"type":"macro","title":"Patch.Assertions.refute_called/1","doc":"Given a call will refute that a matching call was observed by the patched function.\n\nThis macro fully supports patterns.\n\n```elixir\npatch(Example, :function, :patch)\n\nExample.function(1, 2, 3)\n\nPatch.Assertions.refute_called(Example, :function, [4, 5, 6])   # passes\nPatch.Assertions.refute_called(Example, :function, [4, _, 6])  # passes\nPatch.Assertions.refute_called(Example, :function, [1, 2, 3])   # fails\nPatch.Assertions.refute_called(Example, :function, [1, _, 3])  # passes\n```\n\nThere is a convenience macro in the Developer Interface, `Patch.refute_called/1` which should be\npreferred over calling this macro directly.","ref":"Patch.Assertions.html#refute_called/1"},{"type":"macro","title":"Patch.Assertions.refute_called/2","doc":"Given a call will refute that a matching call was observed exactly the number of times provided\nby the patched function.\n\nThis macro fully supports patterns.\n\n```elixir\npatch(Example, :function, :patch)\n\nExample.function(1, 2, 3)\n\nPatch.Assertions.refute_called(Example, :function, [1, 2, 3], 2)   # passes\nPatch.Assertions.refute_called(Example, :function, [1, _, 3], 2)  # passes\n\nExample.function(1, 2, 3)\n\nPatch.Assertions.refute_called(Example, :function, [1, 2, 3], 1)   # passes\nPatch.Assertions.refute_called(Example, :function, [1, _, 3], 1)  # passes\n```\n\nThere is a convenience macro in the Developer Interface, `Patch.refute_called/2` which\nshould be preferred over calling this macro directly.","ref":"Patch.Assertions.html#refute_called/2"},{"type":"macro","title":"Patch.Assertions.refute_called_once/1","doc":"Given a call will refute that a matching call was observed exactly once by the patched function.\n\nThis macro fully supports patterns.\n\n```elixir\npatch(Example, :function, :patch)\n\nExample.function(1, 2, 3)\n\nPatch.Assertions.refute_called_once(Example, :function, [1, 2, 3])   # fails\nPatch.Assertions.refute_called_once(Example, :function, [1, _, 3])  # fails\n\nExample.function(1, 2, 3)\n\nPatch.Assertions.refute_called_once(Example, :function, [1, 2, 3])   # passes\nPatch.Assertions.refute_called_once(Example, :function, [1, _, 3])  # passes\n```\n\nThere is a convenience macro in the Developer Interface, `Patch.refute_called_once/1` which\nshould be preferred over calling this macro directly.","ref":"Patch.Assertions.html#refute_called_once/1"},{"type":"module","title":"Patch.Macro","doc":"","ref":"Patch.Macro.html"},{"type":"function","title":"Patch.Macro.debug/1","doc":"Utility function that acts like `inspect/1` but prints out the Macro as code.","ref":"Patch.Macro.html#debug/1"},{"type":"macro","title":"Patch.Macro.match/2","doc":"Performs an non-hygienic match.\n\nIf the match succeeds true is returned, otherwise a MatchError is raised.\n\nSince the match is non-hygienic pins can be used from the user-scope and binds will effect\nuser-scope.","ref":"Patch.Macro.html#match/2"},{"type":"macro","title":"Patch.Macro.match?/2","doc":"Performs a match, return true if match matches, false otherwise.","ref":"Patch.Macro.html#match?/2"},{"type":"module","title":"Patch.Reflection","doc":"","ref":"Patch.Reflection.html"},{"type":"function","title":"Patch.Reflection.find_arities/2","doc":"","ref":"Patch.Reflection.html#find_arities/2"},{"type":"function","title":"Patch.Reflection.find_functions/1","doc":"","ref":"Patch.Reflection.html#find_functions/1"},{"type":"module","title":"Patch.Supervisor","doc":"","ref":"Patch.Supervisor.html"},{"type":"function","title":"Patch.Supervisor.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Patch.Supervisor.html#child_spec/1"},{"type":"function","title":"Patch.Supervisor.init/1","doc":"","ref":"Patch.Supervisor.html#init/1"},{"type":"function","title":"Patch.Supervisor.start_link/1","doc":"","ref":"Patch.Supervisor.html#start_link/1"},{"type":"exception","title":"Patch.ConfigurationError","doc":"","ref":"Patch.ConfigurationError.html"},{"type":"exception","title":"Patch.InvalidAnyCall","doc":"","ref":"Patch.InvalidAnyCall.html"},{"type":"exception","title":"Patch.MissingCall","doc":"","ref":"Patch.MissingCall.html"},{"type":"exception","title":"Patch.UnexpectedCall","doc":"","ref":"Patch.UnexpectedCall.html"},{"type":"extras","title":"Patch","doc":"# Patch\n\n[![CI](https://github.com/ihumanable/patch/workflows/CI/badge.svg)](https://github.com/ihumanable/patch/actions)\n[![Hex.pm Version](http://img.shields.io/hexpm/v/patch.svg?style=flat)](https://hex.pm/packages/patch)\n[![Hex.pm License](http://img.shields.io/hexpm/l/patch.svg?style=flat)](https://hex.pm/packages/patch)\n[![HexDocs](https://img.shields.io/badge/HexDocs-Yes-blue)](https://hexdocs.pm/patch)\n\nPatch - Ergonomic Mocking for Elixir\n\nPatch makes it easy to replace functionality in tests with test specific functionality.  Patch augments ExUnit with several utilities that make writing tests in Elixir fast and easy.  Patch includes unique functionality that no other mocking library for Elixir provides, Patch's [Super Powers](https://hexdocs.pm/patch/super-powers.html).","ref":"readme.html"},{"type":"extras","title":"Features - Patch","doc":"Why use Patch instead of meck, Mock, Mockery, Mox, etc?  \n\nPatch starts with a very simple idea for how a patched function should work.\n\n> Patched functions should **always** return the mock value they are given.\n\nHere are the key features of Patch.\n\n1. Easy-to-use and composable interface with sensible defaults.\n2. First class support for working with Processes.\n3. No testing code in non-test code.\n\nIn addition to these features which many libraries aspire to, Patch has 3 additional features that no other mocking library for Elixir / Erlang seem to have.  These [Super Powers](https://hexdocs.pm/patch/super-powers.html) are \n\n1. Patch mocks are effective for both local and remote calls.  This means a patched function **always** resolves to the patch.\n2. Patch can patch private functions without changing their visibility.\n3. Patch makes it possible to test your private functions without changing their visibility via the `expose/2` functionality.  \n\nSee the [Mockompare](https://github.com/ihumanable/mockompare) companion project for a comparison of Elixir / Erlang mocking libraries.  If there is a way to accomplish the following with another library, please open an issue so this section and the comparisons can be updated.\n\nFor more information about Patch's Super Powers see the [Super Powers Documentation](https://hexdocs.pm/patch/super-powers.html)","ref":"readme.html#features"},{"type":"extras","title":"Table of Contents - Patch","doc":"- [Installation](#installation)\n- [Quickstart](#quickstart)\n  - [Core Functions](#core-functions)\n  - [Assertions](#assertions)\n  - [Value Builders](#value-builders)\n  - [Customizing Imports](#customizing-imports)\n- [Guide Book](#guide-book)\n- [Support Matrix](#support-matrix)\n- [Limitations](#limitations)\n- [Prior Art](#prior-art)\n- [Changelog](#changelog)","ref":"readme.html#table-of-contents"},{"type":"extras","title":"Installation - Patch","doc":"Add patch to your mix.exs\n\n```elixir\ndef deps do\n  [\n    {:patch, \"~> 0.15.0\", only: [:test]}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Quickstart - Patch","doc":"After adding the dependency just add the following line to any test module after using your test case\n\n```elixir\nuse Patch\n```\n\nThis library comes with a comprehensive suite of unit tests.  These tests not only verify that the library is working correctly but are designed so that for every bit of functionality there is an easy to understand example for how to use that feature.  Check out the [User Tests](https://github.com/ihumanable/patch/tree/master/test/user) for examples of how to use each feature.\n\nUsing Patch adds 11 core functions, 10 assertions, 7 mock value builders, and 1 utility function to the test.  These imports can be controlled, see the [Customizing Imports](#customizing-imports) for details.\n\nSee the [Cheatsheet](https://hexdocs.pm/patch/cheatsheet.html) for an overview of how the library can be used and as a handy reference.  Continue below for links to more in-depth documentation including the [Guidebook](https://hexdocs.pm/patch/01-introduction.html).","ref":"readme.html#quickstart"},{"type":"extras","title":"Core Functions - Patch","doc":"Core functions let us apply patches, patch processes, intercept messages, and query our patched modules.\n\n| Core Function                                                | Description                                                                          |\n|--------------------------------------------------------------|--------------------------------------------------------------------------------------|\n| [expose/2](https://hexdocs.pm/patch/Patch.html#expose/2)     | Expose private functions as public for the purposes of testing                       |\n| [fake/2](https://hexdocs.pm/patch/Patch.html#fake/2)         | Replaces a module with a fake module                                                 |\n| [history/1,2](https://hexdocs.pm/patch/Patch.html#history/2) | Returns the call history for a mock                                                  |\n| [inject/3,4](https://hexdocs.pm/patch/Patch.html#inject/4)   | Injects a listener into a GenServer                                                  |\n| [listen/3](https://hexdocs.pm/patch/Patch.html#listen/3)     | Intercepts messages to a process and forwards them to the test process               |\n| [patch/3](https://hexdocs.pm/patch/Patch.html#patch/3)       | Patches a function so that it returns a mock value                                   |\n| [private/1](https://hexdocs.pm/patch/Patch.html#private/1)   | Macro to call exposed private functions without raising a compiler warning           |\n| [real/1](https://hexdocs.pm/patch/Patch.html#real/1)         | Resolves the real module for a patched module                                        |\n| [replace/3](https://hexdocs.pm/patch/Patch.html#replace/3)   | Replaces part of the state of a GenServer                                            |\n| [restore/1,2](https://hexdocs.pm/patch/Patch.html#restore/1) | Restores an entire module or just a function within a module to its pre-patched form |\n| [spy/1](https://hexdocs.pm/patch/Patch.html#spy/1)           | Patches a module so calls can be asserted without changing behavior                  |","ref":"readme.html#core-functions"},{"type":"extras","title":"Assertions - Patch","doc":"Assertions make it easy to assert that a patched module has or has not observed a call.\n\n| Assertion                                                                        | Description                                                                                                   |\n|----------------------------------------------------------------------------------|---------------------------------------------------------------------------------------------------------------|\n| [assert_any_call/1](https://hexdocs.pm/patch/Patch.html#assert_any_call/1)       | Asserts that any call of any arity has occurred on the mocked module for a function name (preferred macro)    |\n| [assert_any_call/2](https://hexdocs.pm/patch/Patch.html#assert_any_call/2)       | Asserts that any call of any arity has occurred on the mocked module for a function name (advanced use cases) |\n| [assert_called/1](https://hexdocs.pm/patch/Patch.html#assert_called/1)           | Asserts that a particular call has occurred on a mocked module                                                |\n| [assert_called/2](https://hexdocs.pm/patch/Patch.html#assert_called/2)           | Asserts that a particular call has occurred a given number of times on a mocked module                        |\n| [assert_called_once/1](https://hexdocs.pm/patch/Patch.html#assert_called_once/2) | Asserts that a particular call has occurred exactly once on a mocked module                                   |\n| [refute_any_call/1](https://hexdocs.pm/patch/Patch.html#refute_any_call/1)       | Refutes that any call of any arity has occurred on the mocked module for a function name (preferred macro)    |\n| [refute_any_call/2](https://hexdocs.pm/patch/Patch.html#refute_any_call/2)       | Refutes that any call of any arity has occurred on the mocked module for a function name (advanced use cases) |\n| [refute_called/1](https://hexdocs.pm/patch/Patch.html#refute_called/1)           | Refutes that a particular call has occurred on a mocked module                                                |\n| [refute_called/2](https://hexdocs.pm/patch/Patch.html#refute_called/2)           | Refutes that a particular call has occurred a given number of time on a mocked module                         |\n| [refute_called_once/1](https://hexdocs.pm/patch/Patch.html#refute_called_once/1) | Refutes that a particular call has occurred exactly once on a mocked module                                   |","ref":"readme.html#assertions"},{"type":"extras","title":"Value Builders - Patch","doc":"Patched functions aren't limited to only returning simple scalar values, a host of Value Builders are provided for all kinds of testing scenarios.  See the [patch](https://hexdocs.pm/patch/Patch.html#patch/3) documentation for details.\n\n| Value Builder                                                             | Description                                                                                              |\n|---------------------------------------------------------------------------|----------------------------------------------------------------------------------------------------------|\n| [callable/1,2](https://hexdocs.pm/patch/Patch.Mock.Value.html#callable/2) | Callable that will be invoked on every patch invocation, dispatch and evaluation modes can be customized |\n| [cycle/1](https://hexdocs.pm/patch/Patch.Mock.Value.html#cycle/1)         | Cycles through the values provided on every invocation                                                   |\n| [raises/1](https://hexdocs.pm/patch/Patch.Mock.Value.html#raises/1)       | Raises a RuntimeException with the given message upon invocation                                         |\n| [raises/2](https://hexdocs.pm/patch/Patch.Mock.Value.html#raises/2)       | Raises the specified Exception with the given attribtues upon invocation                                 |\n| [scalar/1](https://hexdocs.pm/patch/Patch.Mock.Value.html#scalar/1)       | Returns the argument as a literal, useful for returning functions                                        |\n| [sequence/1](https://hexdocs.pm/patch/Patch.Mock.Value.html#sequence/2)   | Returns the values in order, repeating the last value indefinitely                                       |\n| [throws/1](https://hexdocs.pm/patch/Patch.Mock.Value.html#throws/1)       | Throws the given value upon invocation                                                                   |","ref":"readme.html#value-builders"},{"type":"extras","title":"Utility Functions - Patch","doc":"Patch comes with some utilities that can assist when tests aren't behaving as expected.\n\n| Utility Function                                         | Description                                   |\n|----------------------------------------------------------|-----------------------------------------------|\n| [debug/0,1](https://hexdocs.pm/patch/Patch.html#debug/1) | Enable or Disable debug mode for a given test |","ref":"readme.html#utility-functions"},{"type":"extras","title":"Customizing Imports - Patch","doc":"By default, Patch will import the functions listed in the previous sections.  Imports can be customized through the `:only`, `:except` and `:alias` options.\n\n`:only` and `:except` work similiarly to how they work for the `import` except the values are either a list of symbol atoms or the special atom `:all`.\n\nHere's how only the `expose`, `patch`, and `private` symbols can be imported.\n\n```elixir\nuse Patch, only: [:expose, :patch, :private]\n```\n\nHere's how every symbol except `throws` can be imported\n\n```elixir\nuse Patch, except: [:throws]\n```\n\nPatch also allows you to alias imported symbols, to import `patch` as `mock` the following would be used.\n\n```elixir\nuse Patch, alias: [patch: :mock]\n```","ref":"readme.html#customizing-imports"},{"type":"extras","title":"Guide Book - Patch","doc":"Patch comes with [plenty of documentation](https://hexdocs.pm/patch) and a [Suite of User Tests](https://github.com/ihumanable/patch/tree/master/test/user) that show how to use the library.  \n\nFor a guided tour and deep dive of Patch, see the [Guide Book](https://hexdocs.pm/patch/01-introduction.html)","ref":"readme.html#guide-book"},{"type":"extras","title":"Support Matrix - Patch","doc":"Tests automatically run against a matrix of OTP and Elixir Versions, see the [ci.yml](https://github.com/ihumanable/patch/tree/master/.github/workflows/ci.yml) for details.\n\n| OTP \\ Elixir | 1.9  | 1.10 | 1.11 | 1.12 | 1.13 | 1.14 | 1.15 | 1.16 | 1.17 | 1.18 |\n|:------------:|:----:|:----:|:----:|:----:|:----:|:----:|:----:|:----:|:----:|:----:|\n| 20           | ✅   | N/A  | N/A  | N/A  | N/A  | N/A  | N/A  | N/A  | N/A  | N/A  |\n| 21           | ✅   | ✅   | ✅   | N/A  | N/A  | N/A  | N/A  | N/A  | N/A  | N/A  |\n| 22           | ✅   | ✅   | ✅   | ✅   | ✅   | N/A  | N/A  | N/A  | N/A  | N/A  |\n| 23           | N/A  | ✅   | ✅   | ✅   | ✅   | ✅   | N/A  | N/A  | N/A  | N/A  |\n| 24           | N/A  | N/A  | ✅   | ✅   | ✅   | ✅   | ✅   | N/A  | N/A  | N/A  |\n| 25           | N/A  | N/A  | N/A   | N/A   | ✅   | ✅   | ✅   | ✅   | ✅   | ?    |\n| 26           | N/A  | N/A  | N/A   | N/A   | N/A   | ✅   | ✅   | ✅   | ✅   | ✅   |\n| 27           | N/A  | N/A  | N/A   | N/A   | N/A   | N/A   | N/A   | N/A   | ✅   | ✅   |","ref":"readme.html#support-matrix"},{"type":"extras","title":"Limitations - Patch","doc":"Patch works by recompiling modules, this alters the global execution environment. \n\nSince the global execution environment is altered by Patch, **Patch is not compatible with async: true**.","ref":"readme.html#limitations"},{"type":"extras","title":"Prior Art - Patch","doc":"Up to version 0.5.0 Patch was based off the excellent [meck](https://hex.pm/packages/meck) library.  Patch [Super Powers](https://hexdocs.pm/patch/super-powers.html) required a custom replacement for meck, `Patch.Mock`.  \n\nPatch also takes inspiration from python's [unittest.mock.patch](https://docs.python.org/3/library/unittest.mock.html#patch) for API design.","ref":"readme.html#prior-art"},{"type":"extras","title":"Contributors - Patch","doc":"Patch is made better everyday by developers requesting new features.  \n\n- [daisyzhou](https://github.com/daisyzhou)\n  - Suggested the new function pass through behavior introduced in v0.9.0\n- [likeanocean](https://github.com/likeanocean)\n  - Suggested `assert_called/2`, `assert_called_once/1`, `refute_called/2`, and `refute_called_once/1` introduced in v0.7.0\n- [birarda](https://github.com/birarda)\n  - Suggested `assert_any_call/2`, `refute_any_call/2` introduced in v0.2.0\n  - Suggested `listen/1` introduced in v0.13.0 to listen without a target.\n- [kianmeng](https://github.com/kianmeng)\n  - Corrected several typographical errata\n  - Improved the ci.yml, brining it up to date with best practices.\n- [Dorgan](https://github.com/doorgan)\n  - Reported erratum in the Patch Cheatsheet\n- [Luca Corti](https://github.com/lucacorti)\n  - Reported an issue with warning being emitted by the library on Elixir 1.16 which served as the basis for a bugfix in v0.13.1\n\nIf you have a suggestion for improvements to this library, please open an issue.","ref":"readme.html#contributors"},{"type":"extras","title":"Changelog - Patch","doc":"See the [Changelog](CHANGELOG.md)","ref":"readme.html#changelog"},{"type":"extras","title":"Cheatsheet","doc":"# Cheatsheet\n\nThis cheatsheet provides simple examples for how to use Patch, for more details see the linked documentation.","ref":"cheatsheet.html"},{"type":"extras","title":"Installation - Cheatsheet","doc":"{: .col-2}","ref":"cheatsheet.html#installation"},{"type":"extras","title":"Add Patch to your Dependencies - Cheatsheet","doc":"In the `deps/0` function in the mix.exs file add a line for Patch.\n\n#### mix.exs\n```elixir\ndef deps do\n  [\n    {:patch, \"~> 0.15.0\", only: [:test]}\n  ]\nend\n```","ref":"cheatsheet.html#add-patch-to-your-dependencies"},{"type":"extras","title":"Optionally Including / Excluding Imports - Cheatsheet","doc":"`:only` will cause only a subset of symbols to be imported\n\n#### test/example_only_test.exs\n```elixir\ndefmodule ExampleOnlyTest do\n  use ExUnit.Case\n  use Patch, only: [:expose, :patch, :private]\n\n  # ... snip the rest of the module ...\nend\n```\n\n`:except` will import all symbols except the ones specified\n\n#### test/example_except_test.exs\n```elixir\ndefmodule ExampleExceptTest do\n  use ExUnit.Case\n  use Patch, except: [:fake, :history]\n\n  # ... snip the rest of the module ...\nend\n```","ref":"cheatsheet.html#optionally-including-excluding-imports"},{"type":"extras","title":"Use Patch in your Test Case - Cheatsheet","doc":"In any ExUnit.Case based Test Case add a line to use Patch.\n\n#### test/example_test.exs\n```elixir\ndefmodule ExampleTest do\n  use ExUnit.Case\n  use Patch\n\n  # ... snip the rest of the module ...\nend\n```","ref":"cheatsheet.html#use-patch-in-your-test-case"},{"type":"extras","title":"Aliasing Imports - Cheatsheet","doc":"`:alias` allows the test author to import a symbol while renaming it.\n\n#### test/example_alias_test.exs\n```elixir\ndefmodule ExampleAliasTest do\n  use ExUnit.Case\n  use Patch, alias: [patch: :mock]\n\n  # ... snip the rest of the module ...\nend\n```","ref":"cheatsheet.html#aliasing-imports"},{"type":"extras","title":"Patching - Cheatsheet","doc":"{: .col-2}","ref":"cheatsheet.html#patching"},{"type":"extras","title":"Scalars - Cheatsheet","doc":"```elixir\ntest \"can patch with scalar values\" do\n  assert String.upcase(\"Pre-Patched\") == \"PRE-PATCHED\"\n  \n  patch(String, :upcase, \"PATCHED\")\n\n  assert String.upcase(\"Post-Patched\") == \"PATCHED\"\nend\n```","ref":"cheatsheet.html#scalars"},{"type":"extras","title":"Callables - Cheatsheet","doc":"```elixir\ntest \"can patch with a callable\" do\n  assert String.upcase(\"Pre-Patched\") == \"PRE-PATCHED\"\n\n  patch(String, :upcase, fn s -> String.length(s) end)\n\n  assert String.upcase(\"Post-Patched\") == 12\nend\n```","ref":"cheatsheet.html#callables"},{"type":"extras","title":"Cycles - Cheatsheet","doc":"```elixir\ntest \"can patch with a cycle\" do\n  assert String.upcase(\"Pre-Patched\") == \"PRE-PATCHED\"\n\n  patch(String, :upcase, cycle([1, 2, 3]))\n\n  assert String.upcase(\"Post-Patched\") == 1\n  assert String.upcase(\"Post-Patched\") == 2\n  assert String.upcase(\"Post-Patched\") == 3\n  assert String.upcase(\"Post-Patched\") == 1\n  assert String.upcase(\"Post-Patched\") == 2\nend\n```","ref":"cheatsheet.html#cycles"},{"type":"extras","title":"Sequences - Cheatsheet","doc":"```elixir\ntest \"can patch with a sequence\" do\n  assert String.upcase(\"Pre-Patched\") == \"PRE-PATCHED\"\n\n  patch(String, :upcase, sequence([1, 2, 3]))\n\n  assert String.upcase(\"Post-Patched\") == 1\n  assert String.upcase(\"Post-Patched\") == 2\n  assert String.upcase(\"Post-Patched\") == 3\n  assert String.upcase(\"Post-Patched\") == 3\n  assert String.upcase(\"Post-Patched\") == 3\nend\n```","ref":"cheatsheet.html#sequences"},{"type":"extras","title":"Raises - Cheatsheet","doc":"```elixir\ntest \"can patch to raise a RuntimeError\" do\n  assert String.upcase(\"Pre-Patched\") == \"PRE-PATCHED\"\n  \n  patch(String, :upcase, raises(\"patched\"))\n\n  assert_raise RuntimeError, \"patched\", fn ->\n    String.upcase(\"Post-Patched\")\n  end\nend\n\ntest \"can patch to raise any exception\" do\n  assert String.upcase(\"Pre-Patched\") == \"PRE-PATCHED\"\n  \n  patch(String, :upcase, raises(ArgumentError, message: \"patched\"))\n\n  assert_raise ArgumentError, \"patched\", fn ->\n    String.upcase(\"Post-Patched\")\n  end\nend\n```","ref":"cheatsheet.html#raises"},{"type":"extras","title":"Throws - Cheatsheet","doc":"```elixir\ntest \"can patch to throw a value\" do\n  assert String.upcase(\"Pre-Patched\") == \"PRE-PATCHED\"\n\n  patch(String, :upcase, throws(:patched))\n\n  assert catch_throw(String.upcase(\"Post-Patched\")) == :patched\nend\n```","ref":"cheatsheet.html#throws"},{"type":"extras","title":"Assertions - Cheatsheet","doc":"{: .col-2}","ref":"cheatsheet.html#assertions"},{"type":"extras","title":"assert_called - Cheatsheet","doc":"```elixir\ntest \"can assert calls on patched functions\" do\n  assert String.upcase(\"Pre-Patched\") == \"PRE-PATCHED\"\n  \n  patch(String, :upcase, \"PATCHED\")\n\n  assert String.upcase(\"Post-Patched\") == \"PATCHED\"\n  assert_called String.upcase(\"Post-Patched\")","ref":"cheatsheet.html#assert_called"},{"type":"extras","title":"Arguments can be bound or pattern-matched - Cheatsheet","doc":"assert_called String.upcase(argument)\n  assert argument == \"Post-Patched\"","ref":"cheatsheet.html#arguments-can-be-bound-or-pattern-matched"},{"type":"extras","title":"The number of calls can be specified - Cheatsheet","doc":"assert_called String.upcase(\"Post-Patched\"), 1\nend\n```","ref":"cheatsheet.html#the-number-of-calls-can-be-specified"},{"type":"extras","title":"refute_called - Cheatsheet","doc":"```elixir\ntest \"can refute calls on patched functions\" do\n  assert String.upcase(\"Pre-Patched\") == \"PRE-PATCHED\"\n\n  patch(String, :upcase, \"PATCHED\")\n\n  assert String.upcase(\"Post-Patched\") == \"PATCHED\"\n  refute_called String.upcase(\"Other\")\nend\n```","ref":"cheatsheet.html#refute_called"},{"type":"extras","title":"assert_any_call - Cheatsheet","doc":"```elixir\ntest \"can assert that a patched function was called with any arity\" do\n  assert String.upcase(\"Pre-Patched\") == \"PRE-PATCHED\"\n\n  patch(String, :upcase, \"PATCHED\")\n\n  assert String.upcase(\"Post-Patched\") == \"PATCHED\"\n  assert_any_call String, :upcase\nend\n```","ref":"cheatsheet.html#assert_any_call"},{"type":"extras","title":"refute_any_call - Cheatsheet","doc":"```elixir\ntest \"can refute that a patched function was called with any arity\" do\n  assert String.upcase(\"Pre-Patched\") == \"PRE-PATCHED\"\n  \n  patch(String, :upcase, \"PATCHED\")\n\n  refute_any_call String, :upcase\nend\n```","ref":"cheatsheet.html#refute_any_call"},{"type":"extras","title":"spy - Cheatsheet","doc":"```elixir\ntest \"can assert / refute calls on spied modules without changing behavior\" do\n  spy(String)\n\n  assert String.upcase(\"Example\") == \"EXAMPLE\"\n\n  assert_called String.upcase(\"Example\")\n  refute_called String.upcase(\"Other\")\nend\n```","ref":"cheatsheet.html#spy"},{"type":"extras","title":"history - Cheatsheet","doc":"```elixir\ntest \"can retrieve the list of all calls to a patched module\" do\n  spy(String)\n\n  assert String.upcase(\"Example\") == \"EXAMPLE\"\n  assert String.downcase(\"Example\") == \"example\"\n\n  assert history(String) == [{:upcase, [\"Example\"]}, {:downcase, [\"Example\"]}]\n  assert history(String, :asc) == [{:upcase, [\"Example\"]}, {:downcase, [\"Example\"]}]\n  assert history(String, :desc) == [{:downcase, [\"Example\"]}, {:upcase, [\"Example\"]}]\nend\n```","ref":"cheatsheet.html#history"},{"type":"extras","title":"Private Functions - Cheatsheet","doc":"{: .col-2}","ref":"cheatsheet.html#private-functions"},{"type":"extras","title":"expose - Cheatsheet","doc":"```elixir\ntest \"can expose private functions for testing\" do\n  expose(Example, private_function: 1)\n\n  assert Example.private_function(:argument) == {:ok, :argument}\nend\n```","ref":"cheatsheet.html#expose"},{"type":"extras","title":"private - Cheatsheet","doc":"```elixir\ntest \"can suppress warnings about calling private functions\" do\n  expose(Example, private_function: 1)\n\n  assert private(Example.private_function(:argument)) == {:ok, :argument}\nend\n```","ref":"cheatsheet.html#private"},{"type":"extras","title":"Processes - Cheatsheet","doc":"{: .col-2}","ref":"cheatsheet.html#processes"},{"type":"extras","title":"listen - Cheatsheet","doc":"```elixir\ntest \"can listen to the messages sent to a named process\" do\n  listen(:tag, ExampleNamedProcess)\n\n  send(ExampleNamedProcess, :hello)\n\n  assert_receive {:tag, :hello}\nend\n```\n\n```elixir\ntest \"can listen to the messages sent to a pid\" do\n  pid = Example.start_link()\n\n  listen(:tag, pid)\n\n  send(pid, :hello)\n\n  assert_receive {:tag, :hello}\nend\n```\n\n```elixir\ntest \"can listen to GenServer messages\" do\n  Counter.start_link(0, name: Counter)\n\n  listen(:tag, Counter)\n\n  assert Counter.increment() == 1\n\n  assert_receive {:tag, {GenServer, :call, :increment, from}}  # Bind `from`\n  assert_receive {:tag, {GenServer, :reply, 1, ^from}}         # Match the pinned `from`\nend\n```","ref":"cheatsheet.html#listen"},{"type":"extras","title":"inject - Cheatsheet","doc":"```elixir\ntest \"listeners can be injected into another processes state\" do\n  {:ok, parent_pid} = Parent.start_link()\n\n  inject(:tag, parent_pid, [:child_pid])\n\n  assert Parent.ask_child() == :ok\n\n  assert_recieve {:tag, :ask}\nend\n```","ref":"cheatsheet.html#inject"},{"type":"extras","title":"replace - Cheatsheet","doc":"```elixir\ntest \"process state can be replaced by key\" do\n  {:ok, pid} = Example.start_link()\n\n  assert :sys.get_state(pid).field == :original\n\n  replace(pid, [:field], :updated)\n\n  assert :sys.get_state(pid).field == :updated\nend\n```","ref":"cheatsheet.html#replace"},{"type":"extras","title":"Super Powers","doc":"# Super Powers\n\nPatch provides unique features that no other mocking library for Elixir offers.  See the [Mockompare](https://github.com/ihumanable/mockompare) suite for a comparison of Elixir / Erlang mocking libraries.  If there is a way to accomplish the following with another library, please open an issue so this section and the comparisons can be updated.\n\nSo what are these super powers?\n\n1. Patch mocks are effective for both local and remote calls.  This means a patched function **always** resolves to the patch.\n2. Patch can patch private functions without changing their visibility.\n3. Patch makes it possible to test your private functions without changing their visibility via the `expose/2` functionality.","ref":"super-powers.html"},{"type":"extras","title":"Local and Remote Calls - Super Powers","doc":"In Elixir we have two different ways to call a function, local call vs remote call.\n\nIn a local call, the module is not specified.\n\n```elixir\ndefmodule Example do\n  def example do\n    collaborator()   # No Module, this is a local call\n  end\n\n  def collaborator do\n    :original\n  end\nend\n```\n\nIn a remote call the module is specified, this is most common when calling from one module to another, but can be done within a module if desired.\n\n```elixir\ndefmodule Example do\n  def example do\n    __MODULE__.collaborator()   # Module specified, this is a remote call\n  end\n\n  def collaborator do\n    :original\n  end\nend\n```\n\nIt is exceedingly common to use local calls when writing a module.  The problem comes when mocking a collaborator function.  First, why might we want to mock out a collaborator?  Here's an example where we might want to skip some functionality.\n\n```elixir\ndefmodule Example do\n  def save(thing) do\n    if valid?(thing) do\n      do_save(thing)\n    else\n      {:error, :invalid}\n    end\n  end\n\n  def valid?(thing) do\n    [\n      &Example.Validation.name_not_blank?/1,\n      &Example.Validation.token_hash_valid?/1,\n      &Example.Validation.flux_capacitor_within_limits?/1\n    ]\n    |> Enum.all?(fn validator -> validator.(thing) end)\n  end\n\n  defp do_save(thing) do\n    DB.insert(thing)\n  end\nend\n```\n\nIn the unit tests for `Example.save/1` we want to test the high level logic of \"valid gets saved, invalid gets an error.\"\n\nThis is complicated though because `Example.valid?/1` has real validations baked in.  A common approach is to create a fixture that can pass the validation rules and one that can't.  This is a brittle solution though, it introduces a high degree of coupling between the `Example.save/1` tests and the implementation of `Example.valid?/1`.  \n\nA more robust approach is simply to patch out the call to `Example.valid?/1`.  When we want to test that a valid thing gets saved, we don't have to jump through hoops to get `Example.valid?/1` to return true, we can just patch it and tell it to return true.  When someone comes along and changes the validation rules in `Example.valid?/1` it won't break our `Example.save/1` tests, it might break the tests for `Example.valid?/1` but that's a much better outcome because the test breaking is directly related to the code being changed.\n\nAdditionally, in a unit test we would want to isolate the unit from the database.  Our `Example.do_save/1` method wants to actually write to a database, but this is an implementation detail as far as `Example.save/1` is concerned.  A common approach in unit testing is to replace external dependencies, like APIs and Databases, with Fakes.  \n\nA Fake DB could be as complex another copy of the schema actually running on the real database software that's isolated for test data or as simple as an in memory map.  In this style of testing, the test author can let the code read and write from the fake datastore and then query to make sure the datastore is in the appropriate state.  This approach \"over tests\" the datastore, which is likely already well tested.  A simpler approach is to simply patch out `Example.do_save/1` since we only care that it gets called and it's correct functioning should be guaranteed by tests that directly test that function.\n\nWith Patch, we **can** mock these functions and have the mocks be effective even though the module is using the common local call pattern.\n\n```elixir\ndefmodule ExampleTest do\n  use ExUnit.Case\n  use Patch\n\n  describe \"save/1\" do\n    test \"valid things will be saved\" do\n      # Make everything valid\n      patch(Example, :valid?, true)\n\n      # Patch out do_save so we don't try to hit the database\n      patch(Example, :do_save, :ok)\n\n      assert :ok == Example.save(:thing)\n      assert_called Example.do_save(:thing)\n    end\n\n    test \"invalid things will not be saved\" do\n      # We want to refute the call to do_save/1, so let's spy the entire module\n      spy(Example)\n\n      # Make everything invalid\n      patch(Example, :valid?, false)\n\n      assert {:error, :invalid} == Example.save(:thing)\n      refute_called Example.do_save(_)\n    end\n  end\nend\n```","ref":"super-powers.html#local-and-remote-calls"},{"type":"extras","title":"Patching Private Functions - Super Powers","doc":"Patch allows the test author to patch private functions without doing anything special.  \n\nGiven the following module\n\n```elixir\ndefmodule Example do\n  def public_function(a) do\n    {:ok, private_function(a)}\n  end\n\n  defp private_function(a) do\n    {:private, a}\n  end\nend\n```\n\nWe can write the following test.\n\n```elixir\ndefmodule ExampleTest do\n  use ExUnit.Case\n  use Patch\n\n  test \"public_function/1 wraps private_function/1\" do\n    patch(Example, :private_function, :patched)\n\n    assert Example.public_function(:test_argument) == {:ok, :patched}\n  end\nend\n```\n\nSince Patch guarantees that all calls to a patched function return the mock value, this works as expected.\n\nPrivate functions can be patched just like a public function, but unless they are exposed via `expose/2` they don't become public.  This means that any other code in the project that might have mistakenly made a call to `Example.private_function/1` will fail under test because the visibility is still private.  \n\nTo make a private function public, read on to the next section.","ref":"super-powers.html#patching-private-functions"},{"type":"extras","title":"Testing Private Functions - Super Powers","doc":"Private functions frequently go untested because they are difficult to test.  Developers are faced with a few options when they have a private function.\n\n1.  Don't test the private function.\n2.  Test the private function circuitously by calling some public functions.\n3.  Make a public wrapper for the private function and test that.\n4.  Change the visibility to public and put a comment with some form of, \"This is public just for testing, this function should be treated as though it's private.\"\n\nPatch provides a new mechanism for testing private functions, `expose/2`.\n\nWith `expose/2` the test author can expose any private function to the tests as though it's public.  Here's an example.\n\n```elixir\ndefmodule Example do\n  def public_function(arg) do\n    value = private_function(arg)\n\n    if value < 100 do\n      :small\n    else\n      :large\n    end\n  end","ref":"super-powers.html#testing-private-functions"},{"type":"extras","title":"Private - Super Powers","doc":"defp private_function(arg) when arg < 20 do\n    arg * 1000\n  end\n\n  defp private_function(arg) when arg < 80 do\n    arg - 3\n  end\n\n  defp private_function(arg) do \n    Integer.floor_div(arg, 2)\n  end\nend\n```\n\nTesting `public_function/1` gives us a very coarse measure of if the `private_function/1` logic is working correctly.\n\nIt would be great if we could expose `private_function/1` to be able to more directly test this unit. \n\nHere's how we can expose this function for testing via `expose/2`.  Calling an exposed functions will be flagged by the \nElixir Compiler as a warning, since the exposure happens at runtime not compile-time.  To suppress these warnings, the\n`private/1` macro is provided, just wrap the call to the exposed function with `private/1`.\n\n```elixir\ndefmodule ExampleTest do\n  use ExUnit.Case\n  use Patch\n\n  describe \"private_function/1\" do\n    test \"values less than 20 get magnified by 1000\" do\n      expose(Example, private_function: 1)\n\n      assert private(Example.private_function(10)) == 10_000\n    end\n\n    test \"values between 20 and 80 are reduced by 3\" do\n      expose(Example, private_function: 1)\n\n      assert private(Example.private_function(50)) == 47\n    end\n\n    test \"values greater than or equal to 80 are halved\" do\n      expose(Example, private_function: 1)\n\n      assert private(Example.private_function(120)) == 60\n    end\n  end\nend\n```","ref":"super-powers.html#private"},{"type":"extras","title":"How does this all work? - Super Powers","doc":"Check out the documentation for `Patch.Mock.Code` for more details on how this is accomplished.","ref":"super-powers.html#how-does-this-all-work"},{"type":"extras","title":"Change Log","doc":"# Change Log","ref":"changelog.html"},{"type":"extras","title":"0.15.0 (2024-12-27) - Change Log","doc":"No new functionality in this release but an update to the supported versions.\n\nThanks to [Moosieus](https://github.com/Moosieus) Patch now compiles cleanly on Elixir 1.9+ with the removal of some outdated language constructs in favor of more modern constructs.\n\n0.15.0 removes support for Elixir 1.7 (released in July of 2018) and 1.8 (released in January of 2019).  This version also adds 1.17 and 1.18 to the support matrix.\n\nIf you are still running on 1.7 or 1.8 you should continue to use the 0.14.0 release, if you require ongoing support you should open an issue to discuss.","ref":"changelog.html#0-15-0-2024-12-27"},{"type":"extras","title":"Improvements - Change Log","doc":"- ⬆️ - \\[Versions\\] Elixir 1.17 and 1.18 are now supported and part of CI.","ref":"changelog.html#improvements"},{"type":"extras","title":"Breaking Changes - Change Log","doc":"- 💔 - Elixir 1.7 and Elixir 1.8 are no longer supported.","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"0.14.0 (2024-10-15) - Change Log","doc":"Changes where mocks are evaluated to prevent misuse and allow for common patterns that were not previously supported.\n\nPre-0.14.0 mocks would be intercepted by the `Patch.Mock.Server` and the mock value would be calculated by the server.  This works for most cases, but has surprising behavior when the mock function cares about the process executing the function.  Consider the following example.\n\n```elixir\ndefmodule ExampleTest do\n  use ExUnit.Case\n  use Patch\n\n  test \"example\" do\n    patch(Example, :get_pid, fn -> self() end)\n\n    assert Example.get_pid() == self()\n  end\nend\n```\n\nThis would fail in pre-0.14.0 because the `fn -> self() end` would be executed by the `Patch.Mock.Server` and the pid returned would be the pid for the `Patch.Mock.Server` and not the caller's pid as the test author might expect.\n\n0.14.0 changes this behavior and now will execute the `fn -> self() end` in the caller and return the expected result.  \n\nThis also makes it much more difficult to address the `Patch.Mock.Server` directly, which is generally discouraged as this server is an implementation detail and should only be addressed by the Patch code itself.  This should prevent a class of errors and confusing bugs caused by tests accidentally capturing the pid of, monitoring, or linking to the `Patch.Mock.Server`","ref":"changelog.html#0-14-0-2024-10-15"},{"type":"extras","title":"Improvements - Change Log","doc":"- ⬆️ - \\[Internal\\] Mocks are now evaluated in the caller process instead of the `Patch.Mock.Server` process, see above for details.","ref":"changelog.html#improvements"},{"type":"extras","title":"Breaking Changes - Change Log","doc":"- 💔 - Mocks are now evaluated in the caller process instead of the `Patch.Mock.Server` process.  Using the `Patch.Mock.Server` pid or interacting with the process is not advised but if your tests relied on being able to do this they may break due to this change.","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"0.13.1 (2024-05-02) - Change Log","doc":"Minor bugfix to correct an issue with negative step ranges in `String.slice/2` raised by \n- [Josephine](https://github.com/josephineweidner)\n- [Hissssst](https://github.com/hissssst) \n- [Luca Corti](https://github.com/lucacorti)","ref":"changelog.html#0-13-1-2024-05-02"},{"type":"extras","title":"Bugfixes - Change Log","doc":"- 🐞 - Fixed a warning raised from using a range with a negative step in `String.slice/2`","ref":"changelog.html#bugfixes"},{"type":"extras","title":"0.13.0 (2023-10-17) - Change Log","doc":"Added the ability to control how Patch functions are imported in test modules.  Added the ability to `listen/3` without a target, useful when a process is spawned by another process and the spawning of that process is not within the testing boundaries.","ref":"changelog.html#0-13-0-2023-10-17"},{"type":"extras","title":"Features - Change Log","doc":"- 🎁 - `listen/1` can now be used to construct a listener without a target.\n- 🎁 - `inject/4` can now be used to inject a listener without a target.\n- 🎁 - `use Patch` now accepts `:alias`, `:only`, and `:except` to control exactly which Patch helpers are being pulled in and how they will be named.","ref":"changelog.html#features"},{"type":"extras","title":"Breaking Changes - Change Log","doc":"None","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"0.12.0 (2022-02-08) - Change Log","doc":"Fixed a bug where defective mock functions would be incorrectly classified as unmocked functions, this would engage the passthrough functionality and call the original function.\n\nIf a passthrough mock function's implementation raised `BadArityError` or `FunctionClauseError` it would be incorrectly classified by the mock system as an unmocked function.  The internal mechanisms have been updated to differentiate between exceptions arising directly from the function call vs exceptions in caused by executing the code in the function.\n\nAdded a new `debug/0,1` facility that can be used to enable library level debugging in for a test function.  This functionality can also be controlled by the `:patch` `:debug` configuration value.","ref":"changelog.html#0-12-0-2022-02-08"},{"type":"extras","title":"Features - Change Log","doc":"- 🎁 - `debug/0,1` has been added. By default it will enable debugging for the test it is invoked in.  If debugging has been enabled suite wide via the `:patch` `:debug` configuration value, then `debug/1` can be used with the argument `false` to disable testing for the test it is invoked in.","ref":"changelog.html#features"},{"type":"extras","title":"Bugfixes - Change Log","doc":"- 🐞 - Fixed the issue where defective mocks would cause the mock system to call the original function.","ref":"changelog.html#bugfixes"},{"type":"extras","title":"Improvements - Change Log","doc":"- ⬆️ - \\[CI\\] Updated CI tests from 1.12.2 to 1.12.3 and added 1.13.2 tests to the compatibility matrix.","ref":"changelog.html#improvements"},{"type":"extras","title":"Breaking Changes - Change Log","doc":"None","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"0.11.0 (2022-01-21) - Change Log","doc":"New `private/2` macro to assist with using exposed functions.  \n\nIntroduces \"Tagged Histories\" to prevent a race condition that causes confusing output when using Patch assertions.\n\nRace Condition Description:\n\n1. Assertion function checks the history for a matching call, none is found.\n2. Assertion function pulls the history to format an error message.\n\nBetween 1 and 2 if a matching call arrived then the assertion would fail but print out a message with a matching call.\n\nThis race condition has been defeated by only pulling the history once and using it for both checking for the existence of a call and formatting an error message.\n\nIn addition a bunch of code that was littering Patch.Mock has been moved to a more appropriate location by introducing the concept of a \"Tagged History\" (Patch.Mock.History.Tagged)\n\nTagged Histories have the same entries as a History.  One generates a Tagged History from a History and some matching criteria.  Ever entry in the Tagged History is the entry from the History tagged with a boolean that indicates whether or not it matched the construction criteria.","ref":"changelog.html#0-11-0-2022-01-21"},{"type":"extras","title":"Features - Change Log","doc":"- 🎁 - `private/2` has been added, it's similar to `private/1` but allows the test author to pipe a value into `private/2` and have it end up in the call that's being wrapped.","ref":"changelog.html#features"},{"type":"extras","title":"Bugfixes - Change Log","doc":"- 🐞 - Fixed the behavior of Patch Assertion functions to prevent the race condition described above.","ref":"changelog.html#bugfixes"},{"type":"extras","title":"Breaking Changes - Change Log","doc":"None","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"0.10.2 (2022-01-20) - Change Log","doc":"Major fix to `Patch.Mock.Code.Transforms.Remote`.  Previously this transform completely ignored remote calls.  This would cause an issue when one of the arguments to the remote call was itself a local call.  This has been corrected and the `Patch.Test.User.Patch.LocalCallTest` was updated to prevent regressions.","ref":"changelog.html#0-10-2-2022-01-20"},{"type":"extras","title":"Bugfixes - Change Log","doc":"- 🐞 - Fixed the `Patch.Mock.Code.Transforms.Remote` transformer to correctly handle the arguments of a Remote Call.","ref":"changelog.html#bugfixes"},{"type":"extras","title":"0.10.1 (2021-12-07) - Change Log","doc":"Minor fix to `.formatter.exs`.  Exported format options were not being honored because of a typo, `export` is the honored key but `exports` was being used.","ref":"changelog.html#0-10-1-2021-12-07"},{"type":"extras","title":"Improvements - Change Log","doc":"- ⬆️ - \\[Change Log\\] Removed sections that have no content, except for `Breaking Changes`.  Sections will only be included in the Change Log if some change has actually occurred.  To aid developers upgrading where between versions where breaking changes are allowed, `Breaking Changes` will be included when there are no breaking changes with the description `None` to clearly indicate that no breaking changes have occurred.","ref":"changelog.html#improvements"},{"type":"extras","title":"Bugfixes - Change Log","doc":"- 🐞 - Fixed the `.formatter.exs` so assertion functions won't be parenthesized by projects using `import_deps`","ref":"changelog.html#bugfixes"},{"type":"extras","title":"0.10.0 (2021-12-05) - Change Log","doc":"Changes how function patches work by introducing \"Stacked Callables.\"  \n\nStacked Callables are a large new feature that builds on the passthrough evaluation feature introduced in v0.9.0.\n\n[Chapter 2 of the Guidebook](https://hexdocs.pm/patch/02-patching.html#stacked-callables) has a new section that explains this design in detail.","ref":"changelog.html#0-10-0-2021-12-05"},{"type":"extras","title":"Breaking Changes - Change Log","doc":"- 💔 - Subsequent calls to patch a function with a callable result in the callables stacking.  This may break some tests if the tests rely on one callable completely replacing the previous callable.  Use `restore/2` to clear the manually clear the previous callable.","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Improvements - Change Log","doc":"- ⬆️ - Stacked Callables provide a more ergonomic way to deal with multiple arities than the previous solution of using `:list` dispatch.  See [Stacking and Multiple Arities](https://hexdocs.pm/patch/02-patching.html#stacking-and-multiple-arities)\n- ⬆️ - Stacked Callables provide a more composable way to deal with multiple patches that rely on pattern matching. See [Stacking and Matching](https://hexdocs.pm/patch/02-patching.html#stacking-and-matching)\n- ⬆️ - `callable/2` now allows the caller to configured both the `dispatch` mode and the `evaluation` mode.  This provides a cleaner upgrade path for anyone impacted by the breaking change introduced in v0.9.0.  Using `evaluate: :strict` on a callable will make the callable act like a pre-v0.9.0 callable.","ref":"changelog.html#improvements"},{"type":"extras","title":"Features - Change Log","doc":"- 🎁 - `restore/2` has been added, it's similar to `restore/1` but allows the test author to restore a function in a module instead of the entire module.\n- 🎁 - `callable/2` has a new clause that accepts a `Keyword.t` of options.  Supports `dispatch` which has the current dispatch modes (`:apply`, the default, or `:list`) as well as a new option `evaluate` which accepts either `:passthrough` (the default) or `:strict`.  Strict evaluation behaves like pre-v0.9.0","ref":"changelog.html#features"},{"type":"extras","title":"Deprecations - Change Log","doc":"- ⚠️ - `callable/2` will still accept an `atom` as the second argument.  When it is provided it will be used as the `dispatch` mode and the `evaluate` mode will be set to `passthrough` (the default).  This is a candidate for removal in future versions.","ref":"changelog.html#deprecations"},{"type":"extras","title":"0.9.0 (2021-12-02) - Change Log","doc":"Changes how function patches work so that the test author can only patch out a subset of function calls.","ref":"changelog.html#0-9-0-2021-12-02"},{"type":"extras","title":"Breaking Changes - Change Log","doc":"- 💔 - When patching a function, calls that fail to match the patched function's clauses will passthrough to the original code.  Tests that relied on the old behavior should add a catch-all clause.","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Improvements - Change Log","doc":"- ⬆️ - Improved experience when working with complex functions.  Consider a callback function like `GenServer.handle_call/3`, a test author may wish to only patch out certain messages, allowing other messages to pass through to the original code.  This is now supported, when a patched function fails to match because of either `BadArityError` or `FunctionClauseError` the original code will be called instead.","ref":"changelog.html#improvements"},{"type":"extras","title":"0.8.2 (2021-11-12) - Change Log","doc":"Bugfix for handling module attributes in Call Assertions.","ref":"changelog.html#0-8-2-2021-11-12"},{"type":"extras","title":"Bugfixes - Change Log","doc":"- 🐞 - Fix in `Patch.Macro` to properly handle module attributes when matching.","ref":"changelog.html#bugfixes"},{"type":"extras","title":"0.8.1 (2021-11-12) - Change Log","doc":"Bugfix for handling modules with aggregate compile attributes.  This fixes a codegen bug introduced in 0.8.0.","ref":"changelog.html#0-8-1-2021-11-12"},{"type":"extras","title":"Bugfixes - Change Log","doc":"- 🐞 - Fix in `Patch.Mock.Code.Transforms.Clean` to properly handle aggregate compile attributes.","ref":"changelog.html#bugfixes"},{"type":"extras","title":"0.8.0 (2021-11-11) - Change Log","doc":"Improved call assertion to use full pattern matching.  Pattern matching works like ExUnit's `assert_receive/3` and `assert_received/2`.  Unpinned variables will be bound when asserting.","ref":"changelog.html#0-8-0-2021-11-11"},{"type":"extras","title":"Breaking Changes - Change Log","doc":"- 💔 - Matching has been improved to use full pattern semantics.  Call matching that uses `:_` should be updated to `_`.  Call assertions can now use the full range of Elixir pattern matching.\n- 💔 - `inject/3` has been renamed to `replace/3`","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Improvements - Change Log","doc":"- ⬆️ - Call Assertions now support full pattern matching.\n- ⬆️ - \\[Internal\\] Code Freezer for freezing the modules that Patch uses so test authors can patch modules Patch relies on without breaking the library.","ref":"changelog.html#improvements"},{"type":"extras","title":"Features - Change Log","doc":"- 🎁 - Renamed `inject/3` to `replace/3` which better conveys its functionality\n- 🎁 - Added `inject/4` which injects a listener into a running process.","ref":"changelog.html#features"},{"type":"extras","title":"Bugfixes - Change Log","doc":"- 🐞 - Code Freezer fixes a bug where patching `GenServer` caused Patch to deadlock.","ref":"changelog.html#bugfixes"},{"type":"extras","title":"Removals - Change Log","doc":"- ⛔️ - `inject/3` was removed and renamed to `replace/3`","ref":"changelog.html#removals"},{"type":"extras","title":"0.7.0 (2021-10-21) - Change Log","doc":"Support for call counts in assertions.  `assert_called/1` and `refute_called/1` are unchanged.","ref":"changelog.html#0-7-0-2021-10-21"},{"type":"extras","title":"Breaking Changes - Change Log","doc":"None","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Improvements - Change Log","doc":"- ⬆️ - Exception messages have been improved to clearly indicate which calls have matched.\n- ⬆️ - Assertion Macros have been refactored to minimize injected code in line with Elixir best practices.  Macros now defer to `Patch.Assertions`\n- ⬆️ - Increased test coverage for assertions including improved message formatting.","ref":"changelog.html#improvements"},{"type":"extras","title":"Features - Change Log","doc":"- 🎁 - Added the `assert_any_call/1` macro.  This is now the preferred over `assert_any_call/2`, it allows the test author to write `assert_any_call Module.function` instead of `assert_any_call Module, :function`\n- 🎁 - Added the `assert_called/2` assertion.  The second argument is a call count, this assertion will only pass if there is exactly call count matching calls.\n- 🎁 - Added the `assert_called_once/1` assertion.  This assertion only passes if there is one and only one matching call.\n- 🎁 - Added the `refute_any_call/1` macro.  This is now preferred over `refute_any_call/2`, it allows the test author to write `refute_any_call Module.function` instead of `refute_any_call Module, :function`\n- 🎁 - Added the `refute_called/2` assertion.  The second argument is a call count, this assertion will pass as long as the number of matching calls does not equal the provided call count.\n- 🎁 - Added the `refute_called_once/1` assertion.  This assertion will pass if there are any number of matching calls besides 1.","ref":"changelog.html#features"},{"type":"extras","title":"Deprecations - Change Log","doc":"- ⚠️ - Soft Deprecation for `assert_any_call/2`.  This function is **not** slated for removal but should be reserved for advanced use cases.  Test authors should prefer `assert_any_call/1` when possible.\n- ⚠️ - Soft Deprecation for `refute_any_call/2`.  This function is **not** slated for removal but should be reserved for advanced use cases.  Test authors should prefer `refute_any_call/1` when possible.","ref":"changelog.html#deprecations"},{"type":"extras","title":"0.6.1 (2021-10-17) - Change Log","doc":"Minor release to improve the documentation and reduce the scope of imported symbols from `Patch.Mock.Value`.","ref":"changelog.html#0-6-1-2021-10-17"},{"type":"extras","title":"Improvements - Change Log","doc":"- ⬆️ - \\[Documentation\\] Guide Book broken into Chapters, additional information about core concepts.","ref":"changelog.html#improvements"},{"type":"extras","title":"Removals - Change Log","doc":"- ⛔️ - `Patch.Mock.Value.advance/1` and `Patch.Mock.Value.next/2` used to be imported into the test when `use Patch` was present.  This was an oversight and these two functions are not meant to be called directly by the test author, the imports have been reduced to remove these symbols.","ref":"changelog.html#removals"},{"type":"extras","title":"0.6.0 (2021-10-16) - Change Log","doc":"Major internal refactor.  This version removes `meck` as a dependency and implements a Patch specific replacement, `Patch.Mock`.  This allows us to have a new set of functionality that no other mocking library for Elixir / Erlang has today.  \n\nPatch Mocks can now be said to obey a single simple rule, public or private, local or remote.\n\nA patched function **always** returns the mock value to all callers.\n\nTwo new bits of functionality make this true.\n\n1.  All calls, local or remote, end up intercepted and the mock value returned.\n2.  Private functions can be mocked.\n\nAnd as a bonus\n\n1.  Private functions can be converted into public functions for direct testing.","ref":"changelog.html#0-6-0-2021-10-16"},{"type":"extras","title":"Breaking Changes - Change Log","doc":"- 💔 - Matching semantics have changed since `meck` is no longer the matching engine.  Matching is now literal instead of pseudo-matching, upgrade to version 0.8.0+ for improved matching.","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Improvements - Change Log","doc":"- ⬆️ - \\[Internal\\] `Patch.Mock` introduced to replace `meck`\n- ⬆️ - \\[Documentation\\] README revamped again, new Super Powers documentation and Guide Book.","ref":"changelog.html#improvements"},{"type":"extras","title":"Features - Change Log","doc":"- 🎁 - Added the `expose/2` function to support testing private functions.\n- 🎁 - Added the `history/1,2` function so the history of calls to a mock can be retrieved.\n- 🎁 - Added the `private/1` macro to prevent compiler warnings when calling private functions.\n- 🎁 - Added the `callable/1,2` value builder to create explicit callable mock values.\n- 🎁 - Added the `cycle/1` value builder to create a cycle mock values.\n- 🎁 - Added the `raises/1` value builder to cause a mocked function to raise a RuntimeError.\n- 🎁 - Added the `raises/2` value builder to cause a mocked function to raise any other Exception.\n- 🎁 - Added the `scalar/1` value builder to create explicit scalar mock values.\n- 🎁 - Added the `sequence/1` value builder to create sequence mock values.\n- 🎁 - Added the `throws/1` value builder to cause a mocked function to throw a value.","ref":"changelog.html#features"},{"type":"extras","title":"Removals - Change Log","doc":"- ⛔️ - \\[Dependency\\] `meck` was removed as a dependency","ref":"changelog.html#removals"},{"type":"extras","title":"0.5.0 (2021-09-17) - Change Log","doc":"Better support for mocking erlang modules","ref":"changelog.html#0-5-0-2021-09-17"},{"type":"extras","title":"Breaking Changes - Change Log","doc":"None","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Improvements - Change Log","doc":"- ⬆️ - \\[Internal\\] `patch.release` task to simplify releasing new versions of the library\n- ⬆️ - Support for mocking erlang modules (both sticky and non-sticky)","ref":"changelog.html#improvements"},{"type":"extras","title":"Bugfixes - Change Log","doc":"- 🐞 - Mocking erlang modules actually works now","ref":"changelog.html#bugfixes"},{"type":"extras","title":"0.4.0 (2021-08-09) - Change Log","doc":"Support for working with Processes","ref":"changelog.html#0-4-0-2021-08-09"},{"type":"extras","title":"Breaking Changes - Change Log","doc":"None","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Improvements - Change Log","doc":"- ⬆️ - \\[Testing\\] Testing Matrix updated to latest versions of Elixir / OTP\n- ⬆️ - \\[Dependencies\\] `meck` updated to 0.9.2\n- ⬆️ - \\[Documentation\\] README revamped","ref":"changelog.html#improvements"},{"type":"extras","title":"Features - Change Log","doc":"- 🎁 - Added the `listen/3` function to support listening to a process's messages\n- 🎁 - Added the `inject/3` function to support updating the state of a running process.","ref":"changelog.html#features"},{"type":"extras","title":"0.3.0 (2021-07-12) - Change Log","doc":"Support for replacing a module wholesale via the `fake/2` function","ref":"changelog.html#0-3-0-2021-07-12"},{"type":"extras","title":"Breaking Changes - Change Log","doc":"None","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Improvements - Change Log","doc":"- ⬆️ - [Internal] `Patch.Function.for_arity/2` now accepts an anonymous function it will call instead of a term to return.\n- ⬆️ - [Internal] `Patch.find_functions/1` and `Patch.find_arities/2` use `__info__/1` now instead of doing 256 `function_exported?` checks per function.","ref":"changelog.html#improvements"},{"type":"extras","title":"Features - Change Log","doc":"- 🎁 - Added the `fake/2` function to add support for module fakes.\n- 🎁 - Added the `real/1` function so module fakes can call the real module.","ref":"changelog.html#features"},{"type":"extras","title":"0.2.0 (2021-03-03) - Change Log","doc":"Removed Arity Limitations","ref":"changelog.html#0-2-0-2021-03-03"},{"type":"extras","title":"Breaking Changes - Change Log","doc":"None","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Improvements - Change Log","doc":"- ⬆️ - Removed the arity limitation, can now patch functions of any arity","ref":"changelog.html#improvements"},{"type":"extras","title":"Features - Change Log","doc":"- 🎁 - Added the `assert_any_call/2` and `refute_any_call/2` assertion functions","ref":"changelog.html#features"},{"type":"extras","title":"0.1.2 (2021-01-28) - Change Log","doc":"Increased Elixir Compatibility","ref":"changelog.html#0-1-2-2021-01-28"},{"type":"extras","title":"Improvements - Change Log","doc":"- ⬆️ - Relaxed Elixir version requirement down to 1.7","ref":"changelog.html#improvements"},{"type":"extras","title":"0.1.1 (2020-04-27) - Change Log","doc":"Bugfix Release","ref":"changelog.html#0-1-1-2020-04-27"},{"type":"extras","title":"Improvements - Change Log","doc":"- ⬆️ - Made the library actually work","ref":"changelog.html#improvements"},{"type":"extras","title":"Bugfixes - Change Log","doc":"- 🐞 - Bugfix to make the library actually work","ref":"changelog.html#bugfixes"},{"type":"extras","title":"0.1.0 (2020-04-21) - Change Log","doc":"Initial Release","ref":"changelog.html#0-1-0-2020-04-21"},{"type":"extras","title":"Breaking Changes - Change Log","doc":"None","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Improvements - Change Log","doc":"- ⬆️ - Patch released to the world.  Easy to use and ergonomic Mocking for Elixir.","ref":"changelog.html#improvements"},{"type":"extras","title":"Features - Change Log","doc":"- 🎁 - `patch/3` allows the patching of a module's function with a function.\n- 🎁 - `patch/3` allows the patching of a module's function with a static return value.\n- 🎁 - `spy/1` allows spying on a module.\n- 🎁 - `restore/1` allows removing patches and spies from a module.\n- 🎁 - `assert_called/1` allows for asserting that a patched or spied function has been called with the expected pattern of arguments.\n- 🎁 - `refute_called/1` allows for refuting that a patched or spied function has been called with the expected pattern of arguments.","ref":"changelog.html#features"},{"type":"extras","title":"Chapter 1: Introduction","doc":"# Chapter 1: Introduction\n\nThis guide book will walk you through using all the features and functionality of `Patch` to make great unit tests.","ref":"01-introduction.html"},{"type":"extras","title":"One Big Idea - Chapter 1: Introduction","doc":"Patch is founded on One Big Idea\n\n> Patched functions should **always** return the mock value they are given.\n\nThis is how patched functions behave in `Patch`.  Every function is a valid target for patching and once patched the function will always return the mock value given.  If you ever feel lost, remember this One Big Idea.","ref":"01-introduction.html#one-big-idea"},{"type":"extras","title":"Terminology - Chapter 1: Introduction","doc":"Throughout this guide book we are going to use some common terminology, let's define them.","ref":"01-introduction.html#terminology"},{"type":"extras","title":"Patch - Chapter 1: Introduction","doc":"Patch is used as both a verb and a noun.  To \"patch\" a function is to replace it with an alternative implementation.  The alternative implementation is the noun form of \"patch\" sometimes called the \"patched function.\"  \n\n```elixir\npatch(Example, :example, :patched)\n```\n\n> `Example.example` has been \"patched\" with a \"patch\" that always returns the value `:patched`","ref":"01-introduction.html#patch"},{"type":"extras","title":"Mock Value - Chapter 1: Introduction","doc":"The value returned by a \"patch\" is referred to as the \"mock value\".  There are a number of types of \"mock values\" that will be covered in detail in this guide book.","ref":"01-introduction.html#mock-value"},{"type":"extras","title":"Observed Call - Chapter 1: Introduction","doc":"After a module has been patched, the calls to the module can be observed by `Patch`.  `Patch` comes with utilities to assert or refute that certain calls have been observed.\n\nCalls that happen before the module has been patched are unobserved, the test author can not assert or refute anything about the calls to a module before it has been patched.","ref":"01-introduction.html#observed-call"},{"type":"extras","title":"Chapter 2: Patching","doc":"# Chapter 2: Patching\n\nThe most common operation a test author will perform with `Patch` is, unsurprisingly, patching things.\n\nWhen a module is patched, the patched function will return the mock value provided.","ref":"02-patching.html"},{"type":"extras","title":"Scalar Values - Chapter 2: Patching","doc":"The simplest kind of patch is one that just returns a static scalar value on every invocation.\n\n```elixir\ndefmodule PatchExample do\n  use ExUnit.Case\n  use Patch\n\n  test \"functions can be patched to return a specified value\" do\n    # Assertion passes before patching\n    assert \"HELLO\" == String.upcase(\"hello\")\n\n    # The function can be patched to return a static scalar value\n    patch(String, :upcase, :patched)\n\n    # Assertion passes after patching\n    assert :patched == String.upcase(\"hello\")  \n  end\nend\n```\n\nNo matter how many times we call `String.upcase/1` from here on in and no matter what arguments we pass, we will always get back the value `:patched`.","ref":"02-patching.html#scalar-values"},{"type":"extras","title":"Callable Values - Chapter 2: Patching","doc":"Modules can also be patched to run custom logic instead of returning a static value\n\n```elixir\ndefmodule PatchExample do\n  use ExUnit.Case\n  use Patch\n\n  test \"functions can be patched with a replacement function\" do\n    # Assertion passes before patching\n    assert \"HELLO\" == String.upcase(\"hello\")\n\n    # The function can be patched to run custom code\n    patch(String, :upcase, fn s -> String.length(s) end)\n\n    # Assertion passes after patching\n    assert 5 == String.upcase(\"hello\")\n  end\nend\n```\n\nEvery time we call `String.upcase/1` it will run our function and return the length of the input.","ref":"02-patching.html#callable-values"},{"type":"extras","title":"Passthrough vs Strict Evaluation - Chapter 2: Patching","doc":"By default `Patch` will evaluate the callable in `:passthrough` mode.  In passthrough mode if the callable raises either `BadArityError` or `FunctionClauseError` then the original function will be called.  In `:strict` mode these errors will be returned.\n\nOne of the core design principles underlying `Patch` is that it tries to obey the intent of the test author.  Consider the following example.\n\n```elixir\ndefmodule Example do\n  use GenServer\n\n  # Snip all the GenServer boilerplate\n\n  def handle_call({:a, argument}, _from, state) do\n    # Operation A Definition\n    {:reply, result, state}\n  end\n\n  def handle_call({:b, argument}, _from, state) do\n    # Operation B Definition\n    {:reply, result, state}\n  end\nend\n```\n\nWhen a test author writes the following test code, how should `Patch` interpret it?\n\n```elixir\npatch(Example, :handle_call fn\n  {:a, _argument}, _from, state ->\n    {:reply, :ok, state}\nend)\n```\n\nThere are two possible interpretations, either the test author wants to patch the callback responsible for handling messages shaped like `{:a, argument}` or the test author intends to replace all handle_call callback with one only capable of handling `{:a, argument}`.\n\n`Patch` already has an opinion on which of these to prefer, we can see it in how `Patch` handles patching out one function in a module.  The assumption is that the test author will apply the minimum patch possible, so patching a single function in a module leaves all the other functions in the module as they were.  Applying a similar \"default passthrough\" behavior to this situation it leads us to one conclusion, the test author probably just wants to replace the functionality handling `{:a, argument}` and should leave the other callbacks alone.\n\nThere are always exceptions to the default rule, and `Patch` wants to make it possible to express the other idea.  The test author can inform `Patch` of their intention by using the `:strict` evaluation mode.\n\n```elixir\npatch(Example, :handle_call, callable(fn\n  {:a, _argument}, _from, state ->\n    {:reply, :ok, state}\nend, evaluate: :strict))\n```","ref":"02-patching.html#passthrough-vs-strict-evaluation"},{"type":"extras","title":"Stacked Callables - Chapter 2: Patching","doc":"Callables stack as they are defined in the test.  Every time a function is patched with a callable, that callable is pushed to the top of the stack.  When the patched function is called, the stack is walked from the top to bottom to find a callable that can handle it.\n\nThere are two problems that are nicely solved by Stacked Callables.  Patching functions with multiple arities and making pattern matching in patching composable.\n#### Stacking and Multiple Arities\n\nThe first problem that Stacked Callables solves is patching functions with multiple arities.  Consider this example module.\n\n```elixir\ndefmodule Example do\n  def example(a) do\n    {:original, a}\n  end\n\n  def example(a, b, c) do\n    {:original, a, b, c}\n  end\nend\n```\n\nThis module defines two functions `example/1` and `example/3`.  How can we patch both functions?  Our first attempt might look something like this:\n\n **Note: this code is invalid and won't compile**\n\n```elixir\npatch(Example, :example, fn\n  a ->\n    {:patched, a}\n    \n  a, b, c ->\n    {:patched, a, b, c}\nend)\n```\n\nThis code is illegal in Elixir, the compiler will throw a CompileError and explain that you \"cannot mix clauses with different arities in anonymous functions.\"\n\nThe first solution for this was introduced in v0.6.0 and took the form of a \"dispatch mode.\" By default `Patch` will use the `:apply` mode, which calls the function with the same arity as the patched function was called.  There is an alternative \"dispatch mode\" called `:list` which will pass all the arguments as a single argument, a list of the arguments.\n\n**This code will work, but is unwieldy** \n\n```elixir\npatch(Example, :example, callable(fn \n  [a] ->\n    {:patched, a}\n\n  [a, b, c] ->\n    {:patched, a, b, c}\nend, dispatch: :list)\n```\n\nThis solution made it possible to handle multiple arities, but it is pretty clunky.  With stacked callables we can actually just define two separate callables, one for arity 1, and one for arity 3.\n\n**This code works and is easy to read and write**\n\n```elixir\npatch(Example, :example, fn a -> {:patched, a} end)\npatch(Example, :example, fn a, b, c -> {:patched, a, b, c} end)\n```\n\nTo understand how this works, let's look at how a call to `example/1` and a call to `example/3` work.  The first thing we have to understand is what the Callable Stack looks like, so let's diagram it.\n\n```elixir\n# Top of Stack (latest defined callable)\n[\n  fn a, b, c -> {:patched, a, b, c} end,\n  fn a -> {:patched, a} end\n]\n# Bottom of Stack (earliest defined callable)\n```\n\n`Patch` will run each function until one returns a valid value, the first function to respond with a return value will cause evaluation to complete.\n\nWhen `Example.example(1)` is evaluated it will try the first function.  This function has a different arity so it will raise `BadArityError`, this is one of the two errors that engages passthrough behavior.  Since the first entry in the stack resulted in a logical passthrough `Patch` will try the next entry.  The next entry has the right arity and results in `{:patched, 1}` being returned.  \n\nWhen `Example.example(1, 2, 3)` is evaluated, it's a bit simpler.  The first function is tried and it matches in arity so it immediately returns `{:patched, 1, 2, 3}` and evaluation is completed.\n\n#### Stacking and Matching\n\nAnother problem that Stacked Callables helps solve is composability when patching with pattern matching.  Consider the following example module.\n\n```elixir\ndefmodule Example do\n  def handle(:a) do\n    {:original, :a}\n  end\n\n  def handle(:b) do\n    {:original, :b}\n  end\n\n  def handle(:c) do\n    {:original, :c}\n  end\nend\n```\n\nIf a test author wanted to provide patched behavior for `:a` and `:b` they can do so like this.\n\n```elixir\npatch(Example, :handle, fn\n  :a ->\n    {:patched, :a}\n    \n  :b ->\n    {:patched, :b}\nend)\n```\n\nWhich is a very convenient use of built-in Elixir feature, namely that anonymous functions can have multiple clauses.  But what if we want to have a common behavior for patching out the handling of `:a` and the handling of `:b`.  Perhaps in one test we want patch out `:a`, in another `:a` and `:b`, and in a third just `:b`.  Is there any way that we can DRY up this code and make it composable?\n\nStacked Callables make this quite nice because it makes it possible to patch multiple clauses at different times.  \n\n```elixir\ndefmodule ExampleTest do\n  use ExUnit\n  use Patch\n\n  def patch_a do\n    patch(Example, :handle, fn :a -> {:patched, a} end)\n  end\n\n  def patch_b do\n    patch(Example, :handle, fn :b -> {:patched, b} end)\n  end\n\n  test \"that cares about :a\" do\n    patch_a()\n\n    assert Example.handle(:a) == {:patched, :a}\n  end\n\n  test \"that cares about :a and :b\" do\n    patch_a()\n    patch_b()\n\n    assert Example.handle(:a) == {:patched, :a}\n    assert Example.handle(:b) == {:patched, :b}\n  end\n\n  test \"that cares about :b\" do\n    patch_b()\n\n    assert Example.handle(:b) == {:patched, :b}\n  end\nend\n```\n\nBesides improving composability, it can also just make test code easier to read by breaking multiple logical patches into multiple calls.\n\nCompare\n\n```elixir\npatch(Example, :handle, fn\n  :a ->\n    {:patched, :a}\n    \n  :b ->\n    {:patched, :b}\nend)\n```\n\nwith\n\n```elixir\npatch(Example, :handle, fn :a -> {:patched, :a} end) \npatch(Example, :handle, fn :b -> {:patched, :b} end)\n```\n\nThe power of this mechanism becomes readily apparent when applied to something like GenServer\n\n```elixir\npatch(Example, :handle_call, fn {:a, _args}, _from, state -> {:reply, :ok, state} end)\npatch(Example, :handle_call, fn {:b, _args}, _from, state -> {:reply, :ok, state} end)\n```\n\nIt is common for a GenServer to have many handle_call, handle_cast, and handle_info callbacks.  Being able to define the patches by the pattern makes it easy to patch out a subset of the GenServer's behavior","ref":"02-patching.html#stacked-callables"},{"type":"extras","title":"Functions as Scalars - Chapter 2: Patching","doc":"If functions are always considered callable, how can we patch a function so that it returns a function literal?  This can be accomplished by wrapping the function in a call to `scalar/1` to turn it into a scalar.\n\n```elixir\ndefmodule PatchExample do\n  use ExUnit.Case\n  use Patch\n\n  test \"patch returns a function literal\" do\n    patch(Example, :get_name_normalizer, scalar(&String.downcase/1))\n\n    normalizer = Example.get_name_normalizer()\n    assert normalizer.(\"Patch\") == \"patch\"\n  end\nend\n```","ref":"02-patching.html#functions-as-scalars"},{"type":"extras","title":"Other Values - Chapter 2: Patching","doc":"There are other types of values supported by `Patch`, see [Chapter 3: Mock Values](03-mock-values.html)","ref":"02-patching.html#other-values"},{"type":"extras","title":"Ergonomics - Chapter 2: Patching","doc":"`patch/3` returns the value that the patch will return which can be useful for later on in the test.  Examine this example code for an example\n\n```elixir\ndefmodule PatchExample do\n  use ExUnit.Case\n  use Patch\n\n  test \"patch returns the patch\" do\n    {:ok, expected} = patch(My.Module, :some_function, {:ok, 123})\n\n    # ... additional testing code ...\n\n    assert response.some_function_result == expected\n  end\nend\n```\n\nThis allows the test author to combine creating fixture data with patching.","ref":"02-patching.html#ergonomics"},{"type":"extras","title":"Asserting / Refuting Calls - Chapter 2: Patching","doc":"After a patch is applied, all subsequent calls to the module become \"Observed Calls\" and tests can assert that an expected call has occurred by using the `assert_called/1` macro.\n\n```elixir\ndefmodule PatchExample do\n  use ExUnit.Case\n  use Patch\n\n  test \"asserting calls on a patch\" do\n    patch(String, :upcase, :patched)\n\n    assert :patched = String.upcase(\"hello\")   # Assertion passes after patching\n\n    assert_called String.upcase(\"hello\")       # Assertion passes after call\n  end\nend\n```\n\n`assert_called/1` supports full pattern matching and non-hygienic binds.  This is similar to how ExUnit's `assert_receive/3` and `assert_received/2` work.\n\n```elixir\n# Wildcards are supported\nassert_called String.upcase(_)\n\n# Pinned variables are supported\nexpected = \"hello\"\nassert_called String.upcase(^expected)\n\n# Unpinned variables are supported\nassert_called String.upcase(argument)\nassert argument == \"hello\"\n```\n\nTests can also refute that a call has occurred with the `refute_called/1` macro.  This macro works in much the same way as `assert_called/1` and has full pattern support.\n\n```elixir\ndefmodule PatchExample do\n  use ExUnit.Case\n  use Patch\n\n  test \"refuting calls on a patch\" do\n    patch(String, :upcase, :patched)\n\n    assert \"h\" == String.at(\"hello\", 0)\n\n    refute_called String.upcase(\"hello\")\n  end\nend\n```","ref":"02-patching.html#asserting-refuting-calls"},{"type":"extras","title":"Asserting / Refuting Call Once - Chapter 2: Patching","doc":"We can assert that a call has only happened once with the `assert_called_once/1` macro.  This assertion will only pass if the only one observed call matches.\n\n```elixir\ndefmodule PatchExample do\n  use ExUnit.Case\n  use Patch\n\n  test \"refuting a patch was called once\" do\n    patch(String, :upcase, :patched)\n\n    assert_called_once String.upcase(\"hello\")   # Assertion fails before the function is called.\n\n    assert :patched == String.upcase(\"hello\")\n\n    assert_called_once String.upcase(\"hello\")   # Assertion passes after called once.\n\n    assert :patched == String.upcase(\"hello\")\n\n    assert_called_once String.upcase(\"hello\")   # Assertion fails after second call.\n  end\nend\n```\n\n`assert_called_once/1` supports patterns and binds just like `assert_called/1`.  In the above example the following assertion would behave identically.\n\n```elixir\n# Wildcards are supported\nassert_called_once String.upcase(_)\n\n# Pinned variables are supported\nexpected = \"hello\"\nassert_called_once String.upcase(^expected)\n\n# Unpinned variables are supported\nassert_called_once String.upcase(argument)\nassert argument == \"hello\"\n```\n\nTests can also refute that a call has occurred once with the `refute_called_once/1` macro.  This macro works in much the same way as `assert_called_once/1` and has full pattern support.\n\n```elixir\ndefmodule PatchExample do\n  use ExUnit.Case\n  use Patch\n\n  test \"refuting calls on a patch\" do\n    patch(String, :upcase, :patched)\n\n    refute_called_once String.upcase(\"hello\")   # Assertion passes before the function is called.\n\n    assert :patched == String.upcase(\"hello\")\n\n    refute_called_once String.upcase(\"hello\")   # Assertion fails after called once.\n\n    assert :patched == String.upcase(\"hello\")\n\n    refute_called_once String.upcase(\"hello\")   # Assertion passes after second call.\n  end\nend\n```","ref":"02-patching.html#asserting-refuting-call-once"},{"type":"extras","title":"Asserting / Refuting Call Counts - Chapter 2: Patching","doc":"We can assert that a call has happened some given number of times exactly with the `assert_called/2` macro.  The second argument is the number of observed call matches there must be to pass.\n\n```elixir\ndefmodule PatchExample do\n  use ExUnit.Case\n  use Patch\n\n  test \"asserting 3 calls on a patch\" do\n    patch(String, :upcase, :patched)\n\n    assert :patched == String.upcase(\"hello\")\n\n    assert_called String.upcase(\"hello\"), 3   # Assertion fails after first call.\n    \n    assert :patched == String.upcase(\"hello\")\n\n    assert_called String.upcase(\"hello\"), 3   # Assertion fails after second call.\n    \n    assert :patched == String.upcase(\"hello\")\n\n    assert_called String.upcase(\"hello\"), 3   # Assertion passes after third call.\n  end\nend\n```\n\n`assert_called/2` supports patterns and binds just like `assert_called/1`.  Since multiple calls might match any binds bind to the latest matching call.\n\nIn the above example the following assertion would behave identically.\n\n```elixir\n# Wildcards are supported\nassert_called String.upcase(_), 3\n\n# Pinned variables are supported\nexpected = \"hello\"\nassert_called String.upcase(^expected), 3\n\n# Unpinned variables are supported\nassert_called String.upcase(argument), 3\nassert argument == \"hello\"\n```\n\nTests can also refute that a call has happened an exact number of times with the `refute_called/2` macro.  This macro works in much the same way as `assert_called/2` and also has full pattern support.\n\n```elixir\ndefmodule PatchExample do\n  use ExUnit.Case\n  use Patch\n\n  test \"refuting 3 calls on a patch\" do\n    patch(String, :upcase, :patched)\n\n    assert :patched == String.upcase(\"hello\")\n\n    refute_called String.upcase(\"hello\"), 3   # Assertion passes after first call.\n    \n    assert :patched == String.upcase(\"hello\")\n\n    refute_called String.upcase(\"hello\"), 3   # Assertion passes after second call.\n    \n    assert :patched == String.upcase(\"hello\")\n\n    refute_called String.upcase(\"hello\"), 3   # Assertion fails after third call.\n  end\nend\n```","ref":"02-patching.html#asserting-refuting-call-counts"},{"type":"extras","title":"Asserting / Refuting Multiple Arities - Chapter 2: Patching","doc":"If a function has multiple arities that may be called based on different conditions the test author may wish to assert or refute that a function has been called at all without regards to the number of arguments passed.\n\nThis can be accomplished with the `assert_any_call/1` and `refute_any_call/1` functions.\n\n\n```elixir\ndefmodule PatchExample do\n  use ExUnit.Case\n  use Patch\n\n  test \"asserting any call on a patch\" do\n    patch(String, :pad_leading, fn s -> s end)\n\n    # This formatting call might provide custom padding characters based on\n    # time of day.  (This is an obviously constructed example).\n    TimeOfDaySensitiveFormatter.format(\"Hello World\")\n\n    assert_any_call String.pad_leading\n  end\nend\n```\n\nSimilarly we can refute any call\n\n```elixir\ndefmodule PatchExample do\n  use ExUnit.Case\n  use Patch\n\n  test \"refuting any call on a patch\" do\n    patch(String, :pad_leading, fn s -> s end)\n\n    assert {:error, :not_a_string} = TimeOfDaySensitiveFormatter.format(123)\n\n    refute_any_call String.pad_leading\n  end\nend\n```\n\n#### Advanced Use Cases\n\nThe `assert_any_call/2` and `refute_any_call/2` functions take two arguments the module and the function name as an \natom.  This allows some more advanced use cases where the module or function isn't known at test authoring time.\n\n```elixir\ndefmodule PatchExample\n  use ExUnit.Case\n  use Patch\n\n  test \"asserting any call on normalizer\" do\n    spy(Formatter)\n\n    normalizer = Formatter.get_normalizer()\n\n    assert_any_call Fromatter, normalizer   # Assertion fails before call\n\n    Formatter.normalize(\"hello\", with: normalizer)\n\n    assert_any_call Fromatter, normalizer   # Assertion passes after call\n  end\nend\n```\n\nSimilarly we can refute any call\n\n```elixir\ndefmodule PatchExample\n  use ExUnit.Case\n  use Patch\n\n  test \"refuting any call on normalizer\" do\n    spy(Formatter)\n\n    normalizer = Formatter.get_normalizer()\n\n    refute_any_call Formatter, normalizer   # Assertion passes before call\n\n    Formatter.normalize(\"hello\", with: normalizer)\n\n    refute_any_call Formatter, normalizer   # Assertion fails after call\n  end\nend\n```","ref":"02-patching.html#asserting-refuting-multiple-arities"},{"type":"extras","title":"Chapter 3: Mock Values","doc":"# Chapter 3: Mock Values\n\nIn [Chapter 2: Patching](02-patching.html) we covered two kinds of mock values, Callables and Scalars.\n\nThere are 5 other kinds of mock values available for use in a test.","ref":"03-mock-values.html"},{"type":"extras","title":"Cycle Values - Chapter 3: Mock Values","doc":"Cycle Values will endlessly cycle through a list of return values.  \n\nWhen a patched function has a `Values.Cycle` as its mock value, it will provide the first value in the cycle and then move the first value to the end of the cycle on every invocation.\n\nConsider a function patched with `cycle([1, 2, 3])` via the following code\n\n```elixir\npatch(Example, :example, cycle([1, 2, 3]))\n```\n\n| Invocation | Cycle Before Call | Return Value | Cycle After Call |\n|------------|-------------------|--------------|------------------|\n| 1          | [1, 2, 3]         | 1            | [2, 3, 1]        |\n| 2          | [2, 3, 1]         | 2            | [3, 1, 2]        |\n| 3          | [3, 1, 2]         | 3            | [1, 2, 3]        |\n| 4          | [1, 2, 3]         | 1            | [2, 3, 1]        |\n| 5          | [2, 3, 1]         | 2            | [3, 1, 2]        |\n| 6          | [3, 1, 2]         | 3            | [1, 2, 3]        |\n| 7          | [1, 2, 3]         | 1            | [2, 3, 1]        |\n\nWe could continue the above table forever since the cycle will repeat endlessly.  Cycles can contain `callable/1,2`, `raise/1,2` and `throw/1` mock values.\n\nWe could create a patch that raises a RuntimeError every other call.\n\n```elixir\npatch(Example, :example, cycle([:ok, raises(\"broken\")]))\n```\n\nThis can be helpful for testing retry and backoff constructs, a cycle like this is a good simulation of an unreliable network or dependency.","ref":"03-mock-values.html#cycle-values"},{"type":"extras","title":"Sequence Values - Chapter 3: Mock Values","doc":"Sequence values are similar to cycles, but instead of cycling the list is consumed until only one element is remaining.  Once the sequence has only a single element remaining, that element will be returned on all subsequent calls.\n\nConsider a function patched with `sequence([1, 2, 3])` via the following code\n\n```elixir\npatch(Example, :example, sequence([1, 2, 3]))\n```\n\n| Invocation | Sequence Before Call | Return Value | Sequence After Call |\n|------------|----------------------|--------------|---------------------|\n| 1          | [1, 2, 3]            | 1            | [2, 3]              |\n| 2          | [2, 3]               | 2            | [3]                 |\n| 3          | [3]                  | 3            | [3]                 |\n| 4          | [3]                  | 3            | [3]                 |\n| 5          | [3]                  | 3            | [3]                 |\n\nWe could continue the above table forever since the sequence will continue to return the last value endlessly.  Sequences can contain `callable/1,2`, `raise/1,2` and `throw/1` mock values.\n\nThere is one special behavior of sequence, and that's an empty sequence, which always returns the value `nil` on every invocation.\n\nIf the test author would like to simulate an exhaustable sequence, one that returns a set number of items and then responds to every other call with `nil`, they can simply add a `nil` as the last element in the sequence\n\n```elixir\npatch(Example, :example, sequence([1, 2, 3, nil])\n```\n\n| Invocation | Sequence Before Call | Return Value | Sequence After Call |\n|------------|----------------------|--------------|---------------------|\n| 1          | [1, 2, 3, nil]       | 1            | [2, 3, nil]         |\n| 2          | [2, 3, nil]          | 2            | [3, nil]            |\n| 3          | [3, nil]             | 3            | [nil]               |\n| 4          | [nil]                | nil          | [nil]               |\n| 5          | [nil]                | nil          | [nil]               |","ref":"03-mock-values.html#sequence-values"},{"type":"extras","title":"Raises Value - Chapter 3: Mock Values","doc":"When a function can fail by raising an exception we can use `raises/1,2` to have the patched function raise.\n\n`raise/1` creates a special `Values.Callable` to be used as a mock value.\n\nThis callable ignores the arguments passed in and unconditionally raises a `RuntimeError` with the\ngiven message.\n\n```elixir\npatch(Example, :example, raises(\"patched\"))\n\nassert_raise RuntimeError, \"patched\", fn ->\n  Example.example()\nend\n```\n\n`raise/2` creates a special `Values.Callable` to be used as a mock value.\n\nThis callable ignores the arguments passed in and unconditionally raises the specified exception with the given attributes.\n\n```elixir\npatch(Example, :example, raises(ArgumentError, message: \"patched\"))\n\nassert_raise ArgumentError, \"patched\", fn ->\n  Example.example()\nend\n```","ref":"03-mock-values.html#raises-value"},{"type":"extras","title":"Throws Value - Chapter 3: Mock Values","doc":"When a function can fail by raising an exception we can use `throws/1` to have the patched function throw.\n\n`throws/1` creates a special `Values.Callable` to be used as a mock value.\n\nThis callable ignores the arguments passed in and unconditionally throws the given value.\n\n```elixir\npatch(Example, :example, throws(:patched))\n\nassert catch_throw(Example.example()) == :patched\n```","ref":"03-mock-values.html#throws-value"},{"type":"extras","title":"Chapter 4: Spies and Fakes","doc":"# Chapter 4: Spies and Fakes\n\nIn [Chapter 2: Patching](02-patching.html) and [Chapter 3: Mock Values](03-mock-values.html) we saw how we can patch functions to return particular mock values.\n\nThere are two common cases for patching that have special helpers.","ref":"04-spies-and-fakes.html"},{"type":"extras","title":"Spies - Chapter 4: Spies and Fakes","doc":"If a test wishes to assert / refute calls that happen to a module without actually changing the behavior of the module it can simply `spy/1` the module.  Spies behave identically to the original module but all calls are recorded so `assert_called/1`, `refute_called/1`, `assert_any_called/2`, and `refute_any_called/2` work as expected.\n\n```elixir\ndefmodule PatchExample do\n  use ExUnit.Case\n  use Patch\n\n  def example(value) do\n    String.upcase(value)\n  end\n\n  test \"spies can see what calls happen without changing functionality\" do\n    spy(String)\n\n    assert \"HELLO\" == example(\"hello\")\n\n    assert_called String.upcase(\"hello\")\n  end\nend\n```","ref":"04-spies-and-fakes.html#spies"},{"type":"extras","title":"Fakes - Chapter 4: Spies and Fakes","doc":"Sometimes we want to replace one module with another for testing, for example we might want to replace a module that connects to a real datastore with a fake that stores data in memory while providing the same API.\n\nThe `fake/2,3` functions can be used to replace one module with another.  The replacement module can be completely stand alone or can utilize the functionality of the replaced module, it will be made available through use of the `real/1` function.\n\n```elixir\ndefmodule HighLatencyDatabase do\n  @latency System.convert_time_unit(20, :second, :microsecond)\n\n  def get(id) do\n    {elapsed, response} = :timer.tc(fn -> Patch.real(Database).get(id) end)\n    induce_latency(elapsed)\n    response\n  end\n\n  defp induce_latency(elapsed) when elapsed < @latency do\n    time_to_sleep = System.convert_time_unit(@latency - elapsed, :microsecond, :millisecond)\n    Process.sleep(time_to_sleep)\n  end\n\n  defp induce_latency(_), do: :ok\nend\n```\n\nThis fake module uses the real module to actually get the record from the database and then makes sure that a minimum amount of latency, in this case 20 seconds, is introduced before returning the result.\n\nTo swap out our real Database with our fake HighLatencyDatabase in a test we can now do the following\n\n```elixir\ndefmodule PatchExample do\n  use ExUnit.Case\n  use Patch\n\n  def example(value) do\n    String.upcase(value)\n  end\n\n  test \"API raises TimeoutError when database is experiencing high latency\" do\n    fake(Database, HighLatencyDatabase)\n\n    assert_raises TimeoutError, fn ->\n      API.get(:user, 1)\n    end\n  end\nend\n```","ref":"04-spies-and-fakes.html#fakes"},{"type":"extras","title":"Chapter 5: Processes","doc":"# Chapter 5: Processes\n\nElixir code frequently runs many processes and a test author often wants to assert about the flow of messages between processes.  `Patch` provides some utilities that make listening to the messages between processes easy.","ref":"05-processes.html"},{"type":"extras","title":"Listeners - Chapter 5: Processes","doc":"Listeners are processes that sit between the sender process and the target process.  The listener process will send a copy of every message to the test process so it can use ExUnit's built in `assert_receive`, `assert_received`, `refute_receive`, and `refute_received` functions.\n\nListeners are especially useful when working with named processes since they will automatically unregister the named process and take its place.  For anonymous processes the `inject/3` function is provided to assist in injecting listeners into other processes or the listener can be used in place of the target process when starting consumer processes. \n\nListeners are started with the `listen/3` function and each have a `tag` so that the test process can differentiate which listener has delivered which message.\n\n```elixir\ndefmodule PatchExample do\n  use ExUnit.Case\n  use Patch\n\n  test \"sharded read replication\" do\n    listen(:shard_a_leader, ShardALeader)\n    listen(:shard_a_replica_1, ShardAReplica1)\n    listen(:shard_a_replica_2, ShardAReplica2)\n    \n    listen(:shard_b_leader, ShardBLeader)\n    listen(:shard_b_replica_1, ShardBReplica1)\n    listen(:shard_b_replica_2, ShardBReplica2)\n\n    send(ShardALeader, {:write, :some_value})\n\n    # Assert the leader gets the message\n    assert_receive {:shard_a_leader, {:write, :some_value}}\n\n    # Assert that the replicas for Shard A get the message too\n    assert_receive {:shard_a_replica_1, {:write, :some_value}}\n    assert_receive {:shard_a_replica_2, {:write, :some_value}}\n    \n    # Assert that Shard A does not try to replicate to Shard B\n    refute_receive {:shard_b_leader, {:write, :some_value}}\n    refute_receive {:shard_b_replica_1, {:write, :some_value}}\n    refute_receive {:shard_b_replica_2, {:write, :some_value}}\n  end\nend\n```","ref":"05-processes.html#listeners"},{"type":"extras","title":"GenServer Support - Chapter 5: Processes","doc":"Listeners have special support for GenServers.  By default a listener will provide the test process with all calls, replies, casts, and messages. \n\nGiven a listener with the tag `:tag` the messages from a GenServer are formatted as follows.\n\n| Client Code                   | Message to Test Process                      |\n|:------------------------------|:---------------------------------------------|\n| GenServer.call(pid, :message) | `{:tag, {GenServer, :call, :message, from}}` |\n|  # if capture_replies = true  | `{:tag, {GenServer, :reply, result, from}}`  |\n| GenServer.cast(pid, :message) | `{:tag, {GenServer, :cast, :message}}`       |\n\nDuring a `GenServer.call/3` the listener sits between the client and the server and reports back information to the test process.\n\n```text\n     .------------.          .------.                .--------.                .------.\n     |Test Process|          |client|                |listener|                |server|\n     '------------'          '------'                '--------'                '------'\n           |                    | GenServer.call(message)|                        |    \n           |                    | ----------------------->                        |    \n           |                    |                        |                        |    \n           |      {GenServer, :call, message, from}      |                        |    \n           | <- - - - - - - - - - - - - - - - - - - - - -                         |    \n           |                    |                        |                        |    \n           |                    |                        | GenServer.call(message)|    \n           |                    |                        | ----------------------->    \n           |                    |                        |                        |    \n           |                    |                        |          reply         |    \n           |                    |                        | <-----------------------    \n           |                    |                        |                        |    \n           |       {GenServer, :reply, reply, from}      |                        |    \n           | <- - - - - - - - - - - - - - - - - - - - - -                         |    \n           |                    |                        |                        |    \n           |                    |          reply         |                        |    \n           |                    | <-----------------------                        |    \n     .------------.          .------.                .--------.                .------.\n     |Test Process|          |client|                |listener|                |server|\n     '------------'          '------'                '--------'                '------'`\n```\n\n`GenServer.call/3` allows the client to set a timeout, an amount of time to wait for the server to response.  The listener does not know how long the original client will wait for a timeout, the test author can provide a `:timeout` option when spawning the listener to control how long the listener will wait for its `GenServer.call/3`.  By default the listener will wait 5000ms for each call, the default for `GenServer.call/2`.\n\nSince `assert_receive/3` supports binding, we can use the `from` to match a call and a reply.\n\n```elixir\ndefmodule PatchExample do\n  use ExUnit.Case\n  use Patch\n\n  test \"matching calls and replies\" do\n    Counter.start_link(0, name: Counter)\n\n    listen(:counter, Counter)\n\n    assert Counter.increment() == 1\n\n    assert_receive {:counter, {GenServer, :call, :increment, from}}  # Bind `from`\n    assert_receive {:counter, {GenServer, :reply, 1, ^from}}         # Match the pinned `from`\n  end\nend\n```\n\nIf the test doesn't require the listener to capture replies to `GenServer.call` then the `:capture_replies` option can be set to false.  When this option is false the listener will simply forward the call onto the server.  Refer to the following diagram for details on how this works.\n\n```text\n     .------------.          .------.                .--------.                          .------.\n     |Test Process|          |client|                |listener|                          |server|\n     '------------'          '------'                '--------'                          '------'\n           |                    | GenServer.call(message)|                                  |    \n           |                    | ----------------------->                                  |    \n           |                    |                        |                                  |    \n           |      {GenServer, :call, message, from}      |                                  |    \n           | <- - - - - - - - - - - - - - - - - - - - - -                                   |    \n           |                    |                        |                                  |    \n           |                    |                        | send(:\"$gen_call\", from, message)|    \n           |                    |                        | --------------------------------->    \n           |                    |                        |                                  |    \n           |                    |                        |  reply                           |    \n           |                    | <----------------------------------------------------------    \n     .------------.          .------.                .--------.                          .------.\n     |Test Process|          |client|                |listener|                          |server|\n     '------------'          '------'                '--------'                          '------'\n```","ref":"05-processes.html#genserver-support"},{"type":"extras","title":"Target Monitoring - Chapter 5: Processes","doc":"Listeners will automatically monitor the target process they are listening to.  If the target process goes `:DOWN` the listener will deliver a tagged `{:DOWN, reason}` message to the test process and then exit.","ref":"05-processes.html#target-monitoring"},{"type":"extras","title":"Injecting Listeners - Chapter 5: Processes","doc":"`listen/3` works well for named processes when callers are using the name to send messages to the target process.  What should we do when callers are sending to a pid instead of a name?  This is where the `inject/4` function can be used.\n\n`inject/4` will extract a pid out of a GenServer's state, wrap it with a listener and then replace the pid in the GenServer's state with the listener pid.  `inject/4` will also work with keys in the GenServer's state that are `nil`, it will generate a targetless listener and replace the `nil` with that listner's pid.  This latter behavior is nice when working with a process that will spawn another process and keep track of its pid in state if the spawning behavior is outside the scope of your test.\n\nHere's a simple example, we will have 2 modules, `Target` and `Caller`.\n\n```elixir\ndefmodule Target do\n  use GenServer","ref":"05-processes.html#injecting-listeners"},{"type":"extras","title":"Client - Chapter 5: Processes","doc":"def start_link(multiplier) do\n    GenServer.start_link(__MODULE__, multiplier)\n  end\n\n  def work(pid, argument) do\n    GenServer.call(pid, {:work, argument})\n  end","ref":"05-processes.html#client"},{"type":"extras","title":"Server - Chapter 5: Processes","doc":"def init(multiplier) do\n    {:ok, multiplier}\n  end\n\n  def handle_call({:work, argument}, _from, multiplier) do\n    {:reply, argument * multiplier, multiplier}\n  end\nend\n```\n\nOur `Target` module isn't very interesting, it can do some `work/1` where the caller sends it a number and it multiplies it by the `multiplier` it was started with and returns it.\n\nNext let's look at our `Caller`\n\n```elixir\ndefmodule Caller do\n  use GenServer\n\n  defstruct [:bonus, :target_pid]","ref":"05-processes.html#server"},{"type":"extras","title":"Client - Chapter 5: Processes","doc":"def start_link(bonus, multiplier) do\n    GenServer.start_link(__MODULE__, {bonus, multiplier})\n  end\n\n  def calculate(pid, argument) do\n    GenServer.call(pid, {:calculate, argument})\n  end","ref":"05-processes.html#client"},{"type":"extras","title":"Server - Chapter 5: Processes","doc":"def init({bonus, multiplier}) do\n    {:ok, target_pid} = Target.start_link(multiplier) \n\n    {:ok, %__MODULE__{bonus: bonus, target_pid: target_pid}}\n  end\n\n  def handle_call({:calculate, argument}, _from, %__MODULE__{} = state) do\n    multiplied = Target.work(state.target_pid, argument)\n    {:reply, multiplied + state.bonus, state}\n  end\nend\n```\n\nOur `Caller` takes two values, a `bonus` and a `multiplier`.  It spawns a new `Target` process with the multiplier and stores the `target_pid` in its state.\n\nThe `Caller` process will send a message to the `target_pid` it has stored in its state.  \n\nHere's how we can use `inject/4` to listen to the messages between the `Caller` process and the `Target` process.\n\n```elixir\ndefmodule PatchExample do\n  use ExUnit.Case\n  use Patch\n\n  test \"listen to messages to Target Process\" do\n    bonus = 5\n    multiplier = 10\n\n    {:ok, caller_pid} = Caller.start_link(bonus, multiplier)\n\n    inject(:target, caller_pid, [:target_pid])\n\n    assert Caller.calculate(caller_pid, 7) == 75   # (7 * 10) + 5\n\n    assert_receive {:target, {GenServer, :call, {:work, 7}, from}}\n    assert_receive {:target, {GenServer, :reply, 70, ^from}}\n  end\nend\n```\n\n`inject/4` accepts the same options as `listen/3` and returns the `{:ok, listener_pid}` after successfully injecting the listener.","ref":"05-processes.html#server"},{"type":"extras","title":"Targetless Listeners - Chapter 5: Processes","doc":"Listeners can also be used in place of a real process.  Targetless listeners are nearly identical to listeners with one key exception, `GenServer.call/3`.  A Targetless Listener will happily forward a normal message sent with `send/3` or a `GenServer.cast/2` message to nowhere, but `GenServer.call/3` expects a reply and there isn't one coming when the Listener has no Target.\n\nTo make this failure mode more apparent, since it likely means that the author didn't intend to set up a Targetless Listener, attempting to `GenServer.call/3` at a Targetless Listener will have the following effects.\n\n1. The listener will forward `{tag, {GenServer, :call, message, from}}` to the test process\n2. The listener will forward `{tag, {:EXIT, :no_listener_target}}` to the test process\n3. The listener will crash with the reason `:no_listener_target`","ref":"05-processes.html#targetless-listeners"},{"type":"extras","title":"Replacing State - Chapter 5: Processes","doc":"When working with processes in test code it is sometimes necessary to change the state of a running GenServer.  Common use cases for injecting state into a GenServer are to set up some fixture data, update a configuration value, or replacing pids like in the previous section.\n\n`replace/3` is a helper that handles some common issues when updating state.\n\n```elixir\ndefmodule PatchExample do\n  use ExUnit.Case\n  use Patch\n\n  test \"state can be replaced\" do\n    {:ok, pid} = Target.start_link(:initial_value)\n    \n    assert :initial_value == Target.get_value(pid)\n\n    replace(pid, [:value], :updated_value)\n\n    assert :updated_value == Target.get_value(pid)\n  end\nend\n```\n\n`replace/3` accepts a `GenServer.server` a list of `keys` like one would use for `put_in/3` and then a value to inject into the processes state.\n\nUnlike `put_in/3`, `replace/3` will work with Structs that do not implement the `Access` behaviour.  It does not support the Access functions though, just a list of keys.","ref":"05-processes.html#replacing-state"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}